"""
MCPStore Service Management Module
服务管理相关操作的实现
"""

import asyncio
import logging
import time
from typing import Dict, List, Optional, Any, Union, Tuple

from mcpstore.core.models.service import ServiceConnectionState
from .types import ContextType

logger = logging.getLogger(__name__)


class UpdateServiceAuthHelper:
    """更新服务认证助手 - 明确的服务名，避免状态混乱"""
    
    def __init__(self, context: 'MCPStoreContext', service_name: str, config: Dict[str, Any] = None):
        self._context = context
        self._service_name = service_name  # 🎯 明确的服务名，不会混乱
        self._config = config.copy() if config else {}
    
    def bearer_auth(self, auth: str) -> 'MCPStoreContext':
        """为指定服务更新 Bearer Token 认证"""
        self._config["auth"] = auth
        return self._execute_update()
    
    def custom_headers(self, headers: Dict[str, str]) -> 'MCPStoreContext':
        """为指定服务更新自定义请求头"""
        if "headers" not in self._config:
            self._config["headers"] = {}
        self._config["headers"].update(headers)
        return self._execute_update()
    
    def _execute_update(self) -> 'MCPStoreContext':
        """执行更新服务"""
        self._context._sync_helper.run_async(
            self._context.update_service_async(self._service_name, self._config),
            timeout=60.0
        )
        return self._context


class ServiceManagementMixin:
    """服务管理混入类"""

    def check_services(self) -> dict:
        """
        健康检查（同步版本），store/agent上下文自动判断
        - store上下文：聚合 global_agent_store 下所有 client_id 的服务健康状态
        - agent上下文：聚合 agent_id 下所有 client_id 的服务健康状态
        """
        return self._sync_helper.run_async(self.check_services_async())

    async def check_services_async(self) -> dict:
        """
        异步健康检查，store/agent上下文自动判断
        - store上下文：聚合 global_agent_store 下所有 client_id 的服务健康状态
        - agent上下文：聚合 agent_id 下所有 client_id 的服务健康状态
        """
        if self._context_type.name == 'STORE':
            return await self._store.get_health_status()
        elif self._context_type.name == 'AGENT':
            return await self._store.get_health_status(self._agent_id, agent_mode=True)
        else:
            logger.error(f"[check_services] 未知上下文类型: {self._context_type}")
            return {}

    def get_service_info(self, name: str) -> Any:
        """
        获取服务详情（同步版本），支持 store/agent 上下文
        - store上下文：在 global_agent_store 下的所有 client 中查找服务
        - agent上下文：在指定 agent_id 下的所有 client 中查找服务
        """
        return self._sync_helper.run_async(self.get_service_info_async(name))

    async def get_service_info_async(self, name: str) -> Any:
        """
        获取服务详情（异步版本），支持 store/agent 上下文
        - store上下文：在 global_agent_store 下的所有 client 中查找服务
        - agent上下文：在指定 agent_id 下的所有 client 中查找服务（支持本地名称）
        """
        if not name:
            return {}

        if self._context_type == ContextType.STORE:
            logger.debug(f"STORE mode - searching service in global_agent_store: {name}")
            return await self._store.get_service_info(name)
        elif self._context_type == ContextType.AGENT:
            # Agent模式：将名称原样交给 Store 层处理，Store 负责本地名/全局名的鲁棒解析
            logger.debug(f"AGENT mode - searching service in agent({self._agent_id}): {name}")
            return await self._store.get_service_info(name, self._agent_id)
        else:
            logger.error(f"[get_service_info] 未知上下文类型: {self._context_type}")
            return {}

    def update_service(self, 
                      name: str, 
                      config: Union[Dict[str, Any], None] = None,
                      # 🆕 与 FastMCP 对齐
                      auth: Optional[str] = None,
                      headers: Optional[Dict[str, str]] = None) -> Union['MCPStoreContext', 'UpdateServiceAuthHelper']:
        """
        更新服务配置，支持安全的链式认证
        
        Args:
            name: 服务名称（明确指定，不会混乱）
            config: 新的服务配置
            auth: Bearer token，如果提供则立即执行
            headers: 自定义请求头，如果提供则立即执行
            
        Returns:
            如果有配置或认证参数：立即执行更新，返回 MCPStoreContext
            如果什么都没有：返回 UpdateServiceAuthHelper 支持链式配置
        """
        
        if config is not None:
            # 有配置参数：立即执行更新（保持向后兼容）
            if auth is not None or headers is not None:
                # 配置 + 认证：合并后执行
                final_config = self._apply_auth_to_update_config(config, auth, headers)
            else:
                # 纯配置：直接执行
                final_config = config
                
            self._sync_helper.run_async(
                self.update_service_async(name, final_config),
                timeout=60.0
            )
            return self
        else:
            # 没有配置参数：
            if auth is not None or headers is not None:
                # 纯认证：立即执行
                final_config = self._apply_auth_to_update_config({}, auth, headers)
                self._sync_helper.run_async(
                    self.update_service_async(name, final_config),
                    timeout=60.0
                )
                return self
            else:
                # 什么都没有：返回助手用于链式调用
                return UpdateServiceAuthHelper(self, name, {})

    async def update_service_async(self, name: str, config: Dict[str, Any]) -> bool:
        """
        更新服务配置（异步版本）- 完全替换配置

        Args:
            name: 服务名称
            config: 新的服务配置

        Returns:
            bool: 更新是否成功
        """
        try:
            if self._context_type == ContextType.STORE:
                # Store级别：使用原子更新，避免读改写竞态
                from mcpstore.core.configuration.config_write_service import ConfigWriteService
                cws = ConfigWriteService()
                def _mutator(cfg: Dict[str, Any]) -> Dict[str, Any]:
                    servers = dict(cfg.get("mcpServers", {}))
                    if name not in servers:
                        raise KeyError(f"Service {name} not found in store configuration")
                    servers[name] = config
                    cfg["mcpServers"] = servers
                    return cfg
                try:
                    success = cws.atomic_update(self._store.config.json_path, _mutator)
                except KeyError as e:
                    logger.error(str(e))
                    return False

                if success:
                    # 触发重新注册
                    if hasattr(self._store.orchestrator, 'sync_manager') and self._store.orchestrator.sync_manager:
                        await self._store.orchestrator.sync_manager.sync_global_agent_store_from_mcp_json()

                return success
            else:
                # Agent级别：与单一数据源模式对齐——直接更新 mcp.json 并触发同步
                global_name = name
                if self._service_mapper:
                    global_name = self._service_mapper.to_global_name(name)

                from mcpstore.core.configuration.config_write_service import ConfigWriteService
                cws = ConfigWriteService()
                def _mutator(cfg: Dict[str, Any]) -> Dict[str, Any]:
                    servers = dict(cfg.get("mcpServers", {}))
                    if global_name not in servers:
                        raise KeyError(f"Service {global_name} not found in store configuration (agent mode)")
                    servers[global_name] = config
                    cfg["mcpServers"] = servers
                    return cfg
                try:
                    success = cws.atomic_update(self._store.config.json_path, _mutator)
                except KeyError as e:
                    logger.error(str(e))
                    return False

                if success and hasattr(self._store.orchestrator, 'sync_manager') and self._store.orchestrator.sync_manager:
                    await self._store.orchestrator.sync_manager.sync_global_agent_store_from_mcp_json()

                # 更新缓存中的 metadata.service_config，确保一致性
                try:
                    # 将元数据更新到全局命名空间，保持与生命周期/工具缓存一致
                    global_agent = self._store.client_manager.global_agent_store_id
                    metadata = self._store.registry.get_service_metadata(global_agent, global_name)
                    if metadata:
                        metadata.service_config = config
                        self._store.registry.set_service_metadata(global_agent, global_name, metadata)
                except Exception as _:
                    pass

                return success
        except Exception as e:
            logger.error(f"Failed to update service {name}: {e}")
            return False

    def patch_service(self, name: str, updates: Dict[str, Any]) -> bool:
        """
        增量更新服务配置（同步版本）- 推荐使用

        Args:
            name: 服务名称
            updates: 要更新的配置项

        Returns:
            bool: 更新是否成功
        """
        return self._sync_helper.run_async(self.patch_service_async(name, updates), timeout=60.0)

    async def patch_service_async(self, name: str, updates: Dict[str, Any]) -> bool:
        """
        增量更新服务配置（异步版本）- 推荐使用

        Args:
            name: 服务名称
            updates: 要更新的配置项

        Returns:
            bool: 更新是否成功
        """
        try:
            if self._context_type == ContextType.STORE:
                # Store级别：使用原子增量更新
                from mcpstore.core.configuration.config_write_service import ConfigWriteService
                cws = ConfigWriteService()
                def _mutator(cfg: Dict[str, Any]) -> Dict[str, Any]:
                    servers = dict(cfg.get("mcpServers", {}))
                    if name not in servers:
                        raise KeyError(f"Service {name} not found in store configuration")
                    merged = dict(servers[name])
                    merged.update(updates)
                    servers[name] = merged
                    cfg["mcpServers"] = servers
                    return cfg
                try:
                    success = cws.atomic_update(self._store.config.json_path, _mutator)
                except KeyError as e:
                    logger.error(str(e))
                    return False

                if success:
                    # 触发重新注册
                    if hasattr(self._store.orchestrator, 'sync_manager') and self._store.orchestrator.sync_manager:
                        await self._store.orchestrator.sync_manager.sync_global_agent_store_from_mcp_json()

                return success
            else:
                # Agent级别：与单一数据源模式对齐——直接增量更新 mcp.json 并触发同步
                global_name = name
                if self._service_mapper:
                    global_name = self._service_mapper.to_global_name(name)
                from mcpstore.core.configuration.config_write_service import ConfigWriteService
                cws = ConfigWriteService()
                def _mutator(cfg: Dict[str, Any]) -> Dict[str, Any]:
                    servers = dict(cfg.get("mcpServers", {}))
                    if global_name not in servers:
                        raise KeyError(f"Service {global_name} not found in store configuration (agent mode)")
                    merged = dict(servers[global_name])
                    merged.update(updates)
                    servers[global_name] = merged
                    cfg["mcpServers"] = servers
                    return cfg
                try:
                    success = cws.atomic_update(self._store.config.json_path, _mutator)
                except KeyError as e:
                    logger.error(str(e))
                    return False

                if success and hasattr(self._store.orchestrator, 'sync_manager') and self._store.orchestrator.sync_manager:
                    await self._store.orchestrator.sync_manager.sync_global_agent_store_from_mcp_json()

                # 更新缓存中的 metadata.service_config，确保一致性
                try:
                    # 将元数据更新到全局命名空间，保持与生命周期/工具缓存一致
                    global_agent = self._store.client_manager.global_agent_store_id
                    metadata = self._store.registry.get_service_metadata(global_agent, global_name)
                    if metadata:
                        metadata.service_config.update(updates)
                        self._store.registry.set_service_metadata(global_agent, global_name, metadata)
                except Exception as _:
                    pass

                return success
        except Exception as e:
            logger.error(f"Failed to patch service {name}: {e}")
            return False

    def delete_service(self, name: str) -> bool:
        """
        删除服务（同步版本）

        Args:
            name: 服务名称

        Returns:
            bool: 删除是否成功
        """
        return self._sync_helper.run_async(self.delete_service_async(name), timeout=60.0)

    async def delete_service_async(self, name: str) -> bool:
        """
        删除服务（异步版本，透明代理）

        Args:
            name: 服务名称（Agent 模式下使用本地名称）

        Returns:
            bool: 删除是否成功
        """
        try:
            if self._context_type == ContextType.STORE:
                # Store级别：删除服务并触发双向同步
                await self._delete_store_service_with_sync(name)
                return True
            else:
                # Agent级别：透明代理删除
                await self._delete_agent_service_with_sync(name)
                return True
        except Exception as e:
            logger.error(f"Failed to delete service {name}: {e}")
            return False

    async def delete_service_two_step(self, service_name: str) -> Dict[str, Any]:
        """
        两步删除服务：从配置文件删除 + 从Registry注销

        Args:
            service_name: 服务名称

        Returns:
            Dict: 包含两步操作结果的字典
        """
        result = {
            "step1_config_removal": False,
            "step2_registry_cleanup": False,
            "step1_error": None,
            "step2_error": None,
            "overall_success": False
        }

        # 第一步：从配置文件删除
        try:
            result["step1_config_removal"] = await self.delete_service_async(service_name)
            if not result["step1_config_removal"]:
                result["step1_error"] = "Failed to remove service from configuration"
        except Exception as e:
            result["step1_error"] = f"Configuration removal failed: {str(e)}"
            logger.error(f"Step 1 (config removal) failed: {e}")

        # 第二步：从Registry清理（即使第一步失败也尝试）
        try:
            if self._context_type == ContextType.STORE:
                # Store级别：清理global_agent_store的Registry
                cleanup_success = await self._store.orchestrator.registry.cleanup_service(service_name)
            else:
                # Agent级别：清理特定agent的Registry
                global_name = service_name
                if self._service_mapper:
                    global_name = self._service_mapper.to_global_name(service_name)
                cleanup_success = await self._store.orchestrator.registry.cleanup_service(global_name, self._agent_id)

            result["step2_registry_cleanup"] = cleanup_success
            if not cleanup_success:
                result["step2_error"] = "Failed to cleanup service from registry"
        except Exception as e:
            result["step2_error"] = f"Registry cleanup failed: {str(e)}"
            logger.warning(f"Step 2 (registry cleanup) failed: {e}")

        result["overall_success"] = result["step1_config_removal"] and result["step2_registry_cleanup"]
        return result

    def reset_config(self, scope: str = "all") -> bool:
        """
        重置配置（同步版本）

        Args:
            scope: 重置范围（仅Store级别有效）
                - "all": 重置所有缓存和所有JSON文件（默认）
                - "global_agent_store": 只重置global_agent_store
        """
        return self._sync_helper.run_async(self.reset_config_async(scope), timeout=60.0)

    async def reset_config_async(self, scope: str = "all") -> bool:
        """
        重置配置（异步版本）- 缓存优先模式

        根据上下文类型执行不同的重置操作：
        - Store上下文：根据scope参数重置不同范围
        - Agent上下文：重置该Agent的所有配置（忽略scope参数）

        Args:
            scope: 重置范围（仅Store级别有效）
                - "all": 重置所有缓存和所有JSON文件（默认）
                - "global_agent_store": 只重置global_agent_store
        """
        try:
            if self._context_type == ContextType.STORE:
                return await self._reset_store_config(scope)
            else:
                return await self._reset_agent_config()
        except Exception as e:
            logger.error(f"Failed to reset config: {e}")
            return False

    async def _reset_store_config(self, scope: str) -> bool:
        """Store级别重置配置的内部实现"""
        try:
            if scope == "all":
                logger.debug("Store level: resetting all caches and JSON files")

                # 1. 清空所有缓存
                self._store.registry.agent_clients.clear()
                self._store.registry.client_configs.clear()

                # 清空其他缓存字段
                self._store.registry.sessions.clear()
                self._store.registry.tool_cache.clear()
                self._store.registry.tool_to_session_map.clear()
                self._store.registry.service_states.clear()
                self._store.registry.service_metadata.clear()
                self._store.registry.service_to_client.clear()

                # 2. 重置mcp.json文件
                default_config = {"mcpServers": {}}
                mcp_success = self._store.config.save_config(default_config)

                # 3. 单源模式：不再维护分片映射文件
                logger.debug("Single-source mode: skip shard mapping files (agent_clients/client_services)")

                logger.debug("Store level: all configuration reset completed")
                return mcp_success

            elif scope == "global_agent_store":
                logger.info(" Store级别：只重置global_agent_store")

                # 1. 清空global_agent_store在缓存中的数据
                global_agent_store_id = self._store.client_manager.global_agent_store_id
                self._store.registry.clear(global_agent_store_id)

                # 2. 清空mcp.json文件
                default_config = {"mcpServers": {}}
                mcp_success = self._store.config.save_config(default_config)

                # 3. 单源模式：不再维护分片映射文件
                logger.debug("Single-source mode: skip shard mapping files (agent_clients/client_services)")

                logger.info(" Store级别：global_agent_store重置完成")
                return mcp_success

            else:
                logger.error(f"不支持的scope参数: {scope}")
                return False

        except Exception as e:
            logger.error(f"Store级别重置配置失败: {e}")
            return False

    async def _reset_agent_config(self) -> bool:
        """Agent级别重置配置的内部实现"""
        try:
            logger.info(f" Agent级别：重置Agent {self._agent_id} 的所有配置")

            # 1. 清空Agent在缓存中的数据
            self._store.registry.clear(self._agent_id)

            # 2. 单源模式：不再同步到分片文件
            logger.info("Single-source mode: skip shard mapping files sync")

            logger.info(f" Agent级别：Agent {self._agent_id} 配置重置完成")
            return True

        except Exception as e:
            logger.error(f"Agent级别重置配置失败: {e}")
            return False

    def show_config(self, scope: str = "all") -> Dict[str, Any]:
        """
        显示配置信息（同步版本）

        Args:
            scope: 显示范围（仅Store级别有效）
                - "all": 显示所有Agent的配置（默认）
                - "global_agent_store": 只显示global_agent_store的配置

        Returns:
            Dict: 配置信息字典
        """
        return self._sync_helper.run_async(self.show_config_async(scope), timeout=60.0)

    async def show_config_async(self, scope: str = "all") -> Dict[str, Any]:
        """
        显示配置信息（异步版本）- 从缓存获取

        根据上下文类型执行不同的显示操作：
        - Store上下文：根据scope参数显示不同范围的配置
        - Agent上下文：显示该Agent的配置（忽略scope参数）

        Args:
            scope: 显示范围（仅Store级别有效）
                - "all": 显示所有Agent的配置（默认）
                - "global_agent_store": 只显示global_agent_store的配置

        Returns:
            Dict: 配置信息字典
        """
        try:
            if self._context_type == ContextType.STORE:
                return await self._show_store_config(scope)
            else:
                return await self._show_agent_config()
        except Exception as e:
            logger.error(f"Failed to show config: {e}")
            return {
                "error": f"Failed to show config: {str(e)}",
                "services": {},
                "summary": {"total_services": 0, "total_clients": 0}
            }

    async def _show_store_config(self, scope: str) -> Dict[str, Any]:
        """Store级别显示配置的内部实现"""
        try:
            if scope == "all":
                logger.info(" Store级别：显示所有Agent的配置")

                # 获取所有Agent ID
                all_agent_ids = self._store.registry.get_all_agent_ids()

                agents_config = {}
                total_services = 0
                total_clients = 0

                for agent_id in all_agent_ids:
                    agent_services = {}
                    agent_client_count = 0

                    # 获取该Agent的所有服务
                    service_names = self._store.registry.get_all_service_names(agent_id)

                    for service_name in service_names:
                        complete_info = self._store.registry.get_complete_service_info(agent_id, service_name)
                        client_id = complete_info.get("client_id")
                        config = complete_info.get("config", {})

                        if client_id:
                            agent_services[service_name] = {
                                "client_id": client_id,
                                "config": config
                            }
                            agent_client_count += 1

                    if agent_services:  # 只包含有服务的Agent
                        agents_config[agent_id] = {
                            "services": agent_services
                        }
                        total_services += len(agent_services)
                        total_clients += agent_client_count

                return {
                    "agents": agents_config,
                    "summary": {
                        "total_agents": len(agents_config),
                        "total_services": total_services,
                        "total_clients": total_clients
                    }
                }

            elif scope == "global_agent_store":
                logger.info(" Store级别：只显示global_agent_store的配置")

                global_agent_store_id = self._store.client_manager.global_agent_store_id
                return await self._get_single_agent_config(global_agent_store_id)

            else:
                logger.error(f"不支持的scope参数: {scope}")
                return {
                    "error": f"Unsupported scope parameter: {scope}",
                    "services": {},
                    "summary": {"total_services": 0, "total_clients": 0}
                }

        except Exception as e:
            logger.error(f"Store级别显示配置失败: {e}")
            return {
                "error": f"Failed to show store config: {str(e)}",
                "services": {},
                "summary": {"total_services": 0, "total_clients": 0}
            }

    async def _show_agent_config(self) -> Dict[str, Any]:
        """Agent级别显示配置的内部实现"""
        try:
            logger.info(f" Agent级别：显示Agent {self._agent_id} 的配置")

            # 检查Agent是否存在
            all_agent_ids = self._store.registry.get_all_agent_ids()
            if self._agent_id not in all_agent_ids:
                logger.warning(f"Agent {self._agent_id} not found")
                return {
                    "error": f"Agent '{self._agent_id}' not found",
                    "agent_id": self._agent_id,
                    "services": {},
                    "summary": {"total_services": 0, "total_clients": 0}
                }

            return await self._get_single_agent_config(self._agent_id)

        except Exception as e:
            logger.error(f"Agent级别显示配置失败: {e}")
            return {
                "error": f"Failed to show agent config: {str(e)}",
                "agent_id": self._agent_id,
                "services": {},
                "summary": {"total_services": 0, "total_clients": 0}
            }

    async def _get_single_agent_config(self, agent_id: str) -> Dict[str, Any]:
        """获取单个Agent的配置信息"""
        try:
            services_config = {}
            client_count = 0

            # 获取该Agent的所有服务
            service_names = self._store.registry.get_all_service_names(agent_id)

            for service_name in service_names:
                complete_info = self._store.registry.get_complete_service_info(agent_id, service_name)
                client_id = complete_info.get("client_id")
                config = complete_info.get("config", {})

                if client_id:
                    # Agent级别显示实际的服务名（带后缀的版本）
                    services_config[service_name] = {
                        "client_id": client_id,
                        "config": config
                    }
                    client_count += 1

            return {
                "agent_id": agent_id,
                "services": services_config,
                "summary": {
                    "total_services": len(services_config),
                    "total_clients": client_count
                }
            }

        except Exception as e:
            logger.error(f"获取Agent {agent_id} 配置失败: {e}")
            return {
                "error": f"Failed to get config for agent '{agent_id}': {str(e)}",
                "agent_id": agent_id,
                "services": {},
                "summary": {"total_services": 0, "total_clients": 0}
            }

    def delete_config(self, client_id_or_service_name: str) -> Dict[str, Any]:
        """
        删除服务配置（同步版本）

        Args:
            client_id_or_service_name: client_id或服务名

        Returns:
            Dict: 删除结果
        """
        return self._sync_helper.run_async(self.delete_config_async(client_id_or_service_name), timeout=60.0)

    async def delete_config_async(self, client_id_or_service_name: str) -> Dict[str, Any]:
        """
        删除服务配置（异步版本）

        支持智能参数识别：
        - 如果传入client_id，直接使用
        - 如果传入服务名，自动查找对应的client_id
        - Agent级别严格隔离，只在指定agent范围内查找

        Args:
            client_id_or_service_name: client_id或服务名

        Returns:
            Dict: 删除结果
        """
        try:
            if self._context_type == ContextType.STORE:
                return await self._delete_store_config(client_id_or_service_name)
            else:
                return await self._delete_agent_config(client_id_or_service_name)
        except Exception as e:
            logger.error(f"Failed to delete config: {e}")
            return {
                "success": False,
                "error": f"Failed to delete config: {str(e)}",
                "client_id": None,
                "service_name": None
            }

    def update_config(self, client_id_or_service_name: str, new_config: Dict[str, Any]) -> Dict[str, Any]:
        """
        更新服务配置（同步版本）

        Args:
            client_id_or_service_name: client_id或服务名
            new_config: 新的配置信息

        Returns:
            Dict: 更新结果
        """
        return self._sync_helper.run_async(self.update_config_async(client_id_or_service_name, new_config), timeout=60.0)

    async def update_config_async(self, client_id_or_service_name: str, new_config: Dict[str, Any]) -> Dict[str, Any]:
        """
        更新服务配置（异步版本）

        支持智能参数识别和多种配置格式：
        - 参数识别：client_id或服务名自动识别
        - 配置格式：支持简化格式和mcpServers格式
        - 字段验证：不允许修改服务名，不允许新增字段类型
        - Agent级别严格隔离

        Args:
            client_id_or_service_name: client_id或服务名
            new_config: 新的配置信息

        Returns:
            Dict: 更新结果
        """
        try:
            if self._context_type == ContextType.STORE:
                return await self._update_store_config(client_id_or_service_name, new_config)
            else:
                return await self._update_agent_config(client_id_or_service_name, new_config)
        except Exception as e:
            logger.error(f"Failed to update config: {e}")
            return {
                "success": False,
                "error": f"Failed to update config: {str(e)}",
                "client_id": None,
                "service_name": None,
                "old_config": None,
                "new_config": None
            }

    def _is_deterministic_client_id(self, identifier: str) -> bool:
        """使用 ClientIDGenerator 统一判断确定性client_id格式"""
        try:
            from mcpstore.core.utils.id_generator import ClientIDGenerator
            return ClientIDGenerator.is_deterministic_format(identifier)
        except Exception:
            return False

    def _parse_deterministic_client_id(self, client_id: str, agent_id: str) -> Tuple[str, str]:
        """使用 ClientIDGenerator 统一解析确定性client_id，并验证agent范围"""
        from mcpstore.core.utils.id_generator import ClientIDGenerator
        parsed = ClientIDGenerator.parse_client_id(client_id)
        if parsed.get("type") == "store":
            global_agent_store_id = self._store.client_manager.global_agent_store_id
            if agent_id != global_agent_store_id:
                raise ValueError(f"Store client_id '{client_id}' cannot be used with agent '{agent_id}'")
            return client_id, parsed.get("service_name")
        elif parsed.get("type") == "agent":
            if parsed.get("agent_id") != agent_id:
                raise ValueError(f"Client_id '{client_id}' belongs to agent '{parsed.get('agent_id')}', not '{agent_id}'")
            return client_id, parsed.get("service_name")
        raise ValueError(f"Cannot parse client_id format: {client_id}")

    def _validate_resolved_mapping(self, client_id: str, service_name: str, agent_id: str) -> bool:
        """
        验证解析后的client_id和service_name映射是否有效

        Args:
            client_id: 解析出的client_id
            service_name: 解析出的service_name
            agent_id: Agent ID

        Returns:
            bool: 映射是否有效
        """
        try:
            # 检查client_id是否存在于agent的映射中
            agent_clients = self._store.registry.get_agent_clients_from_cache(agent_id)
            if client_id not in agent_clients:
                logger.debug(f" [VALIDATE_MAPPING] client_id '{client_id}' not found in agent '{agent_id}' clients")
                return False

            # 检查service_name是否存在于Registry中
            existing_client_id = self._store.registry.get_service_client_id(agent_id, service_name)
            if existing_client_id != client_id:
                logger.debug(f" [VALIDATE_MAPPING] service '{service_name}' maps to different client_id: expected={client_id}, actual={existing_client_id}")
                return False

            return True
        except Exception as e:
            logger.debug(f" [VALIDATE_MAPPING] 验证失败: {e}")
            return False

    def _resolve_client_id(self, client_id_or_service_name: str, agent_id: str) -> Tuple[str, str]:
        """
        智能解析client_id或服务名（使用最新的确定性算法）

        Args:
            client_id_or_service_name: 用户输入的参数
            agent_id: Agent ID（用于范围限制）

        Returns:
            Tuple[client_id, service_name]: 解析后的client_id和服务名

        Raises:
            ValueError: 当参数无法解析或不存在时
        """
        logger.debug(f"[RESOLVE_CLIENT_ID] start value='{client_id_or_service_name}' agent='{agent_id}'")

        from .agent_service_mapper import AgentServiceMapper
        global_agent_id = self._store.client_manager.global_agent_store_id

        # 1) 优先：确定性 client_id 直接解析
        if self._is_deterministic_client_id(client_id_or_service_name):
            try:
                client_id, service_name = self._parse_deterministic_client_id(client_id_or_service_name, agent_id)
                logger.debug(f"[RESOLVE_CLIENT_ID] deterministic_ok client_id={client_id} service_name={service_name}")
                return client_id, service_name
            except ValueError as e:
                logger.debug(f"[RESOLVE_CLIENT_ID] deterministic_parse_failed error={e}")
                # 继续按服务名处理

        # 2) Agent 模式：透明代理到 Store（不依赖 Agent 命名空间缓存）
        if self._context_type == ContextType.AGENT and agent_id != global_agent_id:
            # 2.1 判断输入是本地名还是全局名
            input_name = client_id_or_service_name
            global_service_name = None

            if AgentServiceMapper.is_any_agent_service(input_name):
                # 输入是全局名，校验归属
                try:
                    parsed_agent_id, local_name = AgentServiceMapper.parse_agent_service_name(input_name)
                    if parsed_agent_id != agent_id:
                        raise ValueError(f"Service '{input_name}' belongs to agent '{parsed_agent_id}', not '{agent_id}'")
                    global_service_name = input_name
                except ValueError as e:
                    raise ValueError(f"Invalid agent service name '{input_name}': {e}")
            else:
                # 输入是本地名：优先用映射，其次用规则推导
                mapped = self._store.registry.get_global_name_from_agent_service(agent_id, input_name)
                global_service_name = mapped or AgentServiceMapper(agent_id).to_global_name(input_name)

            # 2.2 在 Store 命名空间解析 client_id
            client_id = self._store.registry.get_service_client_id(global_agent_id, global_service_name)
            if not client_id:
                available = ', '.join(self._store.registry.get_all_service_names(global_agent_id)) or 'None'
                raise ValueError(
                    f"Service '{input_name}' (global '{global_service_name}') not found in store. Available services: {available}"
                )

            logger.debug(f"[RESOLVE_CLIENT_ID] agent_proxy_ok local_or_global='{input_name}' -> global='{global_service_name}' client_id={client_id}")
            return client_id, global_service_name

        # 3) Store 模式：直接在 Store 命名空间解析
        service_name = client_id_or_service_name
        service_names = self._store.registry.get_all_service_names(agent_id)
        if service_name in service_names:
            client_id = self._store.registry.get_service_client_id(agent_id, service_name)
            if client_id:
                logger.debug(f"[RESOLVE_CLIENT_ID] store_lookup_ok service={service_name} client_id={client_id}")
                return client_id, service_name
            else:
                raise ValueError(f"Service '{service_name}' found but no client_id mapping")

        available_services = ', '.join(service_names) if service_names else 'None'
        raise ValueError(f"Service '{service_name}' not found in store. Available services: {available_services}")

    async def _delete_store_config(self, client_id_or_service_name: str) -> Dict[str, Any]:
        """Store级别删除配置的内部实现"""
        try:
            logger.info(f"🗑️ Store级别：删除配置 {client_id_or_service_name}")

            global_agent_store_id = self._store.client_manager.global_agent_store_id

            # 解析client_id和服务名
            client_id, service_name = self._resolve_client_id(client_id_or_service_name, global_agent_store_id)

            logger.info(f"🗑️ 解析结果: client_id={client_id}, service_name={service_name}")

            # 验证服务存在
            if not self._store.registry.get_session(global_agent_store_id, service_name):
                logger.warning(f"Service {service_name} not found in registry, but continuing with cleanup")

            # 事务性删除：先删除文件配置，再删除缓存
            # 1. 从mcp.json中删除服务配置
            current_config = self._store.config.load_config()
            if "mcpServers" in current_config and service_name in current_config["mcpServers"]:
                del current_config["mcpServers"][service_name]
                self._store.config.save_config(current_config)
                logger.info(f"🗑️ 已从mcp.json删除服务: {service_name}")

            # 2. 从缓存中删除服务（包括工具和会话）
            self._store.registry.remove_service(global_agent_store_id, service_name)

            # 3. 删除Service-Client映射
            self._store.registry.remove_service_client_mapping(global_agent_store_id, service_name)

            # 4. 删除Client配置
            self._store.registry.remove_client_config(client_id)

            # 5. 删除Agent-Client映射
            self._store.registry.remove_agent_client_mapping(global_agent_store_id, client_id)

            # 6. 单源模式：不再同步到分片文件
            logger.info("Single-source mode: skip shard mapping files sync")

            logger.info(f" Store级别：配置删除完成 {service_name}")

            return {
                "success": True,
                "message": f"Service '{service_name}' deleted successfully",
                "client_id": client_id,
                "service_name": service_name
            }

        except Exception as e:
            logger.error(f"Store级别删除配置失败: {e}")
            return {
                "success": False,
                "error": f"Failed to delete store config: {str(e)}",
                "client_id": None,
                "service_name": None
            }

    async def _delete_agent_config(self, client_id_or_service_name: str) -> Dict[str, Any]:
        """Agent级别删除配置的内部实现"""
        try:
            logger.info(f"🗑️ Agent级别：删除Agent {self._agent_id} 的配置 {client_id_or_service_name}")

            # 解析client_id和服务名
            client_id, service_name = self._resolve_client_id(client_id_or_service_name, self._agent_id)

            logger.info(f"🗑️ 解析结果: client_id={client_id}, service_name={service_name}")

            # 验证服务存在
            if not self._store.registry.get_session(self._agent_id, service_name):
                logger.warning(f"Service {service_name} not found in registry for agent {self._agent_id}, but continuing with cleanup")

            # Agent级别删除：只删除缓存，不修改mcp.json
            # 1. 从缓存中删除服务（包括工具和会话）
            self._store.registry.remove_service(self._agent_id, service_name)

            # 2. 删除Service-Client映射
            self._store.registry.remove_service_client_mapping(self._agent_id, service_name)

            # 3. 删除Client配置
            self._store.registry.remove_client_config(client_id)

            # 4. 删除Agent-Client映射
            self._store.registry.remove_agent_client_mapping(self._agent_id, client_id)

            # 5. 单源模式：不再同步到分片文件
            logger.info("Single-source mode: skip shard mapping files sync")

            logger.info(f" Agent级别：配置删除完成 {service_name}")

            return {
                "success": True,
                "message": f"Service '{service_name}' deleted successfully from agent '{self._agent_id}'",
                "client_id": client_id,
                "service_name": service_name
            }

        except Exception as e:
            logger.error(f"Agent级别删除配置失败: {e}")
            return {
                "success": False,
                "error": f"Failed to delete agent config: {str(e)}",
                "client_id": None,
                "service_name": None
            }

    def _validate_and_normalize_config(self, new_config: Dict[str, Any], service_name: str, old_config: Dict[str, Any]) -> Dict[str, Any]:
        """
        验证和标准化配置

        Args:
            new_config: 新配置
            service_name: 服务名
            old_config: 原配置

        Returns:
            Dict: 标准化后的配置

        Raises:
            ValueError: 配置验证失败
        """
        # 1. 处理配置格式
        if "mcpServers" in new_config:
            # mcpServers格式
            if len(new_config["mcpServers"]) != 1:
                raise ValueError("mcpServers format must contain exactly one service")

            config_service_name = list(new_config["mcpServers"].keys())[0]
            if config_service_name != service_name:
                raise ValueError(f"Cannot change service name from '{service_name}' to '{config_service_name}'")

            normalized_config = new_config["mcpServers"][service_name]
        else:
            # 简化格式
            if "name" in new_config:
                raise ValueError("Cannot modify service name in config update")
            normalized_config = new_config.copy()

        # 2. 验证字段类型一致性
        old_config_keys = set(old_config.keys())
        new_config_keys = set(normalized_config.keys())

        # 检查是否有新增的字段类型
        new_fields = new_config_keys - old_config_keys
        if new_fields:
            raise ValueError(f"Cannot add new field types: {list(new_fields)}. Only existing fields can be updated.")

        # 3. 验证字段值的合理性
        for key, value in normalized_config.items():
            if key in old_config:
                old_type = type(old_config[key])
                new_type = type(value)

                # 允许的类型转换
                if old_type != new_type:
                    # 允许字符串和数字之间的转换
                    if not ((old_type in [str, int, float] and new_type in [str, int, float]) or
                            (old_type == list and new_type == list)):
                        raise ValueError(f"Field '{key}' type mismatch: expected {old_type.__name__}, got {new_type.__name__}")

        return normalized_config

    async def _update_store_config(self, client_id_or_service_name: str, new_config: Dict[str, Any]) -> Dict[str, Any]:
        """Store级别更新配置的内部实现"""
        try:
            logger.info(f" Store级别：更新配置 {client_id_or_service_name}")

            global_agent_store_id = self._store.client_manager.global_agent_store_id

            # 解析client_id和服务名
            client_id, service_name = self._resolve_client_id(client_id_or_service_name, global_agent_store_id)

            logger.info(f" 解析结果: client_id={client_id}, service_name={service_name}")

            # 获取当前配置
            old_complete_info = self._store.registry.get_complete_service_info(global_agent_store_id, service_name)
            old_config = old_complete_info.get("config", {})

            if not old_config:
                raise ValueError(f"Service '{service_name}' configuration not found")

            # 验证和标准化新配置
            normalized_config = self._validate_and_normalize_config(new_config, service_name, old_config)

            logger.info(f" 配置验证通过，开始更新: {service_name}")

            # 1. 清空服务的工具和会话数据
            self._store.registry.clear_service_tools_only(global_agent_store_id, service_name)

            # 2. 更新Client配置缓存
            self._store.registry.update_client_config(client_id, {
                "mcpServers": {service_name: normalized_config}
            })

            # 3. 设置服务状态为INITIALIZING并更新元数据
            from mcpstore.core.models.service import ServiceConnectionState
            self._store.registry.set_service_state(global_agent_store_id, service_name, ServiceConnectionState.INITIALIZING)

            # 更新服务元数据中的配置
            metadata = self._store.registry.get_service_metadata(global_agent_store_id, service_name)
            if metadata:
                metadata.service_config = normalized_config
                metadata.consecutive_failures = 0
                metadata.error_message = None
                from datetime import datetime
                metadata.state_entered_time = datetime.now()
                self._store.registry.set_service_metadata(global_agent_store_id, service_name, metadata)

            # 4. 更新mcp.json文件
            current_config = self._store.config.load_config()
            if "mcpServers" not in current_config:
                current_config["mcpServers"] = {}
            current_config["mcpServers"][service_name] = normalized_config
            self._store.config.save_config(current_config)

            # 5. 单源模式：不再同步到分片文件
            logger.info("Single-source mode: skip shard mapping files sync")

            # 6. 触发生命周期管理器重新初始化服务
            self._store.orchestrator.lifecycle_manager.initialize_service(
                global_agent_store_id, service_name, normalized_config
            )

            logger.info(f" Store级别：配置更新完成 {service_name}")

            return {
                "success": True,
                "message": f"Service '{service_name}' configuration updated successfully",
                "client_id": client_id,
                "service_name": service_name,
                "old_config": old_config,
                "new_config": normalized_config
            }

        except Exception as e:
            logger.error(f"Store级别更新配置失败: {e}")
            return {
                "success": False,
                "error": f"Failed to update store config: {str(e)}",
                "client_id": None,
                "service_name": None,
                "old_config": None,
                "new_config": None
            }

    async def _update_agent_config(self, client_id_or_service_name: str, new_config: Dict[str, Any]) -> Dict[str, Any]:
        """Agent级别更新配置的内部实现"""
        try:
            logger.info(f" Agent级别：更新Agent {self._agent_id} 的配置 {client_id_or_service_name}")

            # 解析client_id和服务名
            client_id, service_name = self._resolve_client_id(client_id_or_service_name, self._agent_id)

            logger.info(f" 解析结果: client_id={client_id}, service_name={service_name}")

            # 获取当前配置
            old_complete_info = self._store.registry.get_complete_service_info(self._agent_id, service_name)
            old_config = old_complete_info.get("config", {})

            if not old_config:
                raise ValueError(f"Service '{service_name}' configuration not found")

            # 验证和标准化新配置
            normalized_config = self._validate_and_normalize_config(new_config, service_name, old_config)

            logger.info(f" 配置验证通过，开始更新: {service_name}")

            # 1. 清空服务的工具和会话数据
            self._store.registry.clear_service_tools_only(self._agent_id, service_name)

            # 2. 更新Client配置缓存
            self._store.registry.update_client_config(client_id, {
                "mcpServers": {service_name: normalized_config}
            })

            # 3. 设置服务状态为INITIALIZING并更新元数据
            from mcpstore.core.models.service import ServiceConnectionState
            self._store.registry.set_service_state(self._agent_id, service_name, ServiceConnectionState.INITIALIZING)

            # 更新服务元数据中的配置
            metadata = self._store.registry.get_service_metadata(self._agent_id, service_name)
            if metadata:
                metadata.service_config = normalized_config
                metadata.consecutive_failures = 0
                metadata.error_message = None
                from datetime import datetime
                metadata.state_entered_time = datetime.now()
                self._store.registry.set_service_metadata(self._agent_id, service_name, metadata)

            # 4. 单源模式：不再同步到分片文件（Agent级别不更新mcp.json）
            logger.info("Single-source mode: skip shard mapping files sync")

            # 5. 触发生命周期管理器重新初始化服务
            self._store.orchestrator.lifecycle_manager.initialize_service(
                self._agent_id, service_name, normalized_config
            )

            logger.info(f" Agent级别：配置更新完成 {service_name}")

            return {
                "success": True,
                "message": f"Service '{service_name}' configuration updated successfully for agent '{self._agent_id}'",
                "client_id": client_id,
                "service_name": service_name,
                "old_config": old_config,
                "new_config": normalized_config
            }

        except Exception as e:
            logger.error(f"Agent级别更新配置失败: {e}")
            return {
                "success": False,
                "error": f"Failed to update agent config: {str(e)}",
                "client_id": None,
                "service_name": None,
                "old_config": None,
                "new_config": None
            }

    def get_service_status(self, name: str) -> dict:
        """获取单个服务的状态信息（同步版本）"""
        return self._sync_helper.run_async(self.get_service_status_async(name))

    async def get_service_status_async(self, name: str) -> dict:
        """获取单个服务的状态信息"""
        try:
            if self._context_type == ContextType.STORE:
                return self._store.orchestrator.get_service_status(name)
            else:
                # Agent模式：转换服务名称
                global_name = name
                if self._service_mapper:
                    global_name = self._service_mapper.to_global_name(name)
                # 透明代理：在全局命名空间查询状态
                return self._store.orchestrator.get_service_status(global_name)
        except Exception as e:
            logger.error(f"Failed to get service status for {name}: {e}")
            return {"status": "error", "error": str(e)}

    def restart_service(self, name: str) -> bool:
        """重启指定服务（同步版本）"""
        return self._sync_helper.run_async(self.restart_service_async(name))

    async def restart_service_async(self, name: str) -> bool:
        """重启指定服务（透明代理）"""
        try:
            if self._context_type == ContextType.STORE:
                return await self._store.orchestrator.restart_service(name)
            else:
                # Agent模式：透明代理 - 将本地服务名映射到全局服务名，并在全局命名空间执行重启
                global_name = await self._map_agent_service_to_global(name)
                global_agent = self._store.client_manager.global_agent_store_id
                return await self._store.orchestrator.restart_service(global_name, global_agent)
        except Exception as e:
            logger.error(f"Failed to restart service {name}: {e}")
            return False

    # ===  新增：Agent 透明代理辅助方法 ===

    async def _map_agent_service_to_global(self, local_name: str) -> str:
        """
        将 Agent 的本地服务名映射到全局服务名

        Args:
            local_name: Agent 中的本地服务名

        Returns:
            str: 全局服务名
        """
        try:
            if self._agent_id:
                # 尝试从映射关系中获取全局名称
                global_name = self._store.registry.get_global_name_from_agent_service(self._agent_id, local_name)
                if global_name:
                    logger.debug(f" [SERVICE_PROXY] 服务名映射: {local_name} → {global_name}")
                    return global_name

            # 如果映射失败，可能是 Store 原生服务，直接返回
            logger.debug(f" [SERVICE_PROXY] 无映射，使用原名: {local_name}")
            return local_name

        except Exception as e:
            logger.error(f" [SERVICE_PROXY] 服务名映射失败: {e}")
            return local_name

    async def _delete_store_service_with_sync(self, service_name: str):
        """Store 服务删除（带双向同步）"""
        try:
            # 1. 从 Registry 中删除
            self._store.registry.remove_service(
                self._store.client_manager.global_agent_store_id,
                service_name
            )

            # 2. 从 mcp.json 中删除
            current_config = self._store.config.load_config()
            if "mcpServers" in current_config and service_name in current_config["mcpServers"]:
                del current_config["mcpServers"][service_name]
                success = self._store.config.save_config(current_config)

                if success:
                    logger.info(f" [SERVICE_DELETE] Store 服务删除成功: {service_name}")
                else:
                    logger.error(f" [SERVICE_DELETE] Store 服务删除失败: {service_name}")

            # 3. 触发双向同步（如果是 Agent 服务）
            if hasattr(self._store, 'bidirectional_sync_manager'):
                await self._store.bidirectional_sync_manager.handle_service_deletion_with_sync(
                    self._store.client_manager.global_agent_store_id,
                    service_name
                )

        except Exception as e:
            logger.error(f" [SERVICE_DELETE] Store 服务删除失败 {service_name}: {e}")
            raise

    async def _delete_agent_service_with_sync(self, local_name: str):
        """Agent 服务删除（带双向同步）"""
        try:
            # 1. 获取全局名称
            global_name = self._store.registry.get_global_name_from_agent_service(self._agent_id, local_name)
            if not global_name:
                logger.warning(f" [SERVICE_DELETE] 未找到映射关系: {self._agent_id}:{local_name}")
                return

            # 2. 从 Agent 缓存中删除
            self._store.registry.remove_service(self._agent_id, local_name)

            # 3. 从 Store 缓存中删除
            self._store.registry.remove_service(
                self._store.client_manager.global_agent_store_id,
                global_name
            )

            # 4. 移除映射关系
            self._store.registry.remove_agent_service_mapping(self._agent_id, local_name)

            # 5. 从 mcp.json 中删除
            current_config = self._store.config.load_config()
            if "mcpServers" in current_config and global_name in current_config["mcpServers"]:
                del current_config["mcpServers"][global_name]
                success = self._store.config.save_config(current_config)

                if success:
                    logger.info(f" [SERVICE_DELETE] Agent 服务删除成功: {local_name} → {global_name}")
                else:
                    logger.error(f" [SERVICE_DELETE] Agent 服务删除失败: {local_name} → {global_name}")

            # 6. 单源模式：不再同步到分片文件
            logger.info("Single-source mode: skip shard mapping files sync")

        except Exception as e:
            logger.error(f" [SERVICE_DELETE] Agent 服务删除失败 {self._agent_id}:{local_name}: {e}")
            raise

    def show_mcpconfig(self) -> Dict[str, Any]:
        """
        根据当前上下文（store/agent）获取对应的配置信息

        Returns:
            Dict[str, Any]: Store上下文返回MCP JSON格式，Agent上下文返回client配置字典
        """
        if self._context_type == ContextType.STORE:
            # Store上下文：返回MCP JSON格式的配置
            try:
                config = self._store.config.load_config()
                # 确保返回格式正确
                if isinstance(config, dict) and 'mcpServers' in config:
                    return config
                else:
                    logger.warning("Invalid MCP config format")
                    return {"mcpServers": {}}
            except Exception as e:
                logger.error(f"Failed to show MCP config: {e}")
                return {"mcpServers": {}}
        else:
            # Agent上下文：返回所有相关client配置的字典
            agent_id = self._agent_id
            client_ids = self._store.registry.get_agent_clients_from_cache(agent_id)

            # 获取每个client的配置
            result = {}
            for client_id in client_ids:
                client_config = self._store.orchestrator.client_manager.get_client_config(client_id)
                if client_config:
                    result[client_id] = client_config

            return result

    def wait_service(self, client_id_or_service_name: str,
                    status: Union[str, List[str]] = 'healthy',
                    timeout: float = 10.0,
                    raise_on_timeout: bool = False) -> bool:
        """
        等待服务达到指定状态（同步版本）

        Args:
            client_id_or_service_name: client_id或服务名（智能识别）
            status: 目标状态，可以是单个状态字符串或状态列表
            timeout: 超时时间（秒），默认10秒
            raise_on_timeout: 超时时是否抛出异常，默认False

        Returns:
            bool: 成功达到目标状态返回True，超时返回False

        Raises:
            TimeoutError: 当raise_on_timeout=True且超时时抛出
            ValueError: 当参数无法解析时抛出
        """
        return self._sync_helper.run_async(
            self.wait_service_async(client_id_or_service_name, status, timeout, raise_on_timeout),
            timeout=timeout + 1.0  # 给异步版本额外1秒缓冲
        )

    async def wait_service_async(self, client_id_or_service_name: str,
                               status: Union[str, List[str]] = 'healthy',
                               timeout: float = 10.0,
                               raise_on_timeout: bool = False) -> bool:
        """
        等待服务达到指定状态（异步版本）

        Args:
            client_id_or_service_name: client_id或服务名（智能识别）
            status: 目标状态，可以是单个状态字符串或状态列表
            timeout: 超时时间（秒），默认10秒
            raise_on_timeout: 超时时是否抛出异常，默认False

        Returns:
            bool: 成功达到目标状态返回True，超时返回False

        Raises:
            TimeoutError: 当raise_on_timeout=True且超时时抛出
            ValueError: 当参数无法解析时抛出
        """
        try:
            # 解析参数
            agent_scope = self._agent_id if self._context_type == ContextType.AGENT else self._store.client_manager.global_agent_store_id
            client_id, service_name = self._resolve_client_id(client_id_or_service_name, agent_scope)

            # 在纯视图模式下，Agent 的状态查询统一使用全局命名空间
            status_agent_key = self._store.client_manager.global_agent_store_id

            # 解析等待模式
            change_mode = False
            if isinstance(status, str) and status.lower() == 'change':
                change_mode = True
                logger.info(f"[WAIT_SERVICE] start mode=change service='{service_name}' timeout={timeout}s")
                initial_status = self._store.orchestrator.get_service_comprehensive_status(service_name, status_agent_key)
            else:
                # 规范化目标状态
                target_statuses = self._normalize_target_statuses(status)
                logger.info(f"[WAIT_SERVICE] start mode=target service='{service_name}' client_id='{client_id}' target={target_statuses} timeout={timeout}s")

            start_time = time.time()
            poll_interval = 0.2  # 200ms轮询间隔
            prev_status = None
            last_log = start_time

            while True:
                # 检查超时
                elapsed = time.time() - start_time
                if elapsed >= timeout:
                    if change_mode:
                        msg = f"[WAIT_SERVICE] timeout mode=change service='{service_name}' from='{initial_status}' elapsed={elapsed:.2f}s"
                    else:
                        msg = f"[WAIT_SERVICE] timeout mode=target service='{service_name}' target={target_statuses} last='{prev_status}' elapsed={elapsed:.2f}s"
                    logger.warning(msg)
                    if raise_on_timeout:
                        raise TimeoutError(msg)
                    return False

                # 获取当前状态
                try:
                    current_status = self._store.orchestrator.get_service_comprehensive_status(service_name, status_agent_key)

                    # 仅在状态变化或每2秒节流一次打印
                    now = time.time()
                    if current_status != prev_status or (now - last_log) > 2.0:
                        logger.debug(f"[WAIT_SERVICE] status service='{service_name}' value='{current_status}'")
                        prev_status, last_log = current_status, now

                    if change_mode:
                        if current_status != initial_status:
                            logger.info(f"[WAIT_SERVICE] done mode=change service='{service_name}' from='{initial_status}' to='{current_status}' elapsed={elapsed:.2f}s")
                            return True
                    else:
                        # 检查是否达到目标状态
                        if current_status in target_statuses:
                            logger.info(f"[WAIT_SERVICE] done mode=target service='{service_name}' reached='{current_status}' elapsed={elapsed:.2f}s")
                            return True
                except Exception as e:
                    # 降级到 debug，避免无意义刷屏
                    logger.debug(f"[WAIT_SERVICE] status_error service='{service_name}' error={e}")
                    # 继续轮询

                # 等待下次轮询
                await asyncio.sleep(poll_interval)

        except ValueError as e:
            logger.error(f"[WAIT_SERVICE] param_error error={e}")
            raise
        except Exception as e:
            logger.error(f"[WAIT_SERVICE] unexpected_error error={e}")
            if raise_on_timeout:
                raise
            return False

    def _normalize_target_statuses(self, status: Union[str, List[str]]) -> List[str]:
        """
        规范化目标状态参数

        Args:
            status: 状态参数，可以是字符串或列表

        Returns:
            List[str]: 规范化的状态列表

        Raises:
            ValueError: 当状态值无效时抛出
        """
        # 获取有效的状态值
        valid_statuses = {state.value for state in ServiceConnectionState}

        if isinstance(status, str):
            target_statuses = [status]
        elif isinstance(status, list):
            target_statuses = status
        else:
            raise ValueError(f"Status must be string or list, got {type(status)}")

        # 验证状态值
        for s in target_statuses:
            if s not in valid_statuses:
                raise ValueError(f"Invalid status '{s}'. Valid statuses are: {sorted(valid_statuses)}")

        return target_statuses

    def _apply_auth_to_update_config(self, config: Dict[str, Any], 
                                    auth: Optional[str], 
                                    headers: Optional[Dict[str, str]]) -> Dict[str, Any]:
        """将认证配置应用到更新配置中"""
        final_config = config.copy() if config else {}
        
        if auth is not None:
            final_config["auth"] = auth
        
        if headers is not None:
            if "headers" not in final_config:
                final_config["headers"] = {}
            final_config["headers"].update(headers)
        
        return final_config





"""
MCPOrchestrator Resources and Prompts Module
Resources/Prompts模块 - 包含FastMCP的Resources和Prompts功能支持
"""

import logging
import time
from typing import Dict, Any, Optional

from mcpstore.core.utils.mcp_client_helpers import temp_client_for_service

logger = logging.getLogger(__name__)

class ResourcesPromptsMixin:
    """Resources/Prompts混入类"""

    # === 工具变更检测接口 ===

    def list_changed_tools(
        self,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None,
        force_refresh: bool = False
    ) -> Dict[str, Any]:
        """
        工具变更检测和处理方法（同步版本）

        Args:
            service_name: 特定服务名（可选，None表示检查所有服务）
            client_id: 特定客户端ID（可选）
            force_refresh: 是否强制刷新（忽略缓存和时间间隔）

        Returns:
            Dict: 包含变更信息的响应
            {
                "changed": bool,           # 是否有变更
                "services": List[str],     # 发生变更的服务列表
                "trigger": str,           # 触发方式："notification" | "polling" | "manual"
                "timestamp": str,         # 检测时间
                "details": Dict           # 详细变更信息
            }
        """
        if self.tools_update_monitor:
            return self.tools_update_monitor.list_changed_tools(
                service_name=service_name,
                client_id=client_id,
                force_refresh=force_refresh,
                trigger="manual"
            )
        else:
            logger.warning("ToolsUpdateMonitor not available")
            return {
                "changed": False,
                "services": [],
                "trigger": "manual",
                "timestamp": self._get_timestamp(),
                "details": {"error": "ToolsUpdateMonitor not available"}
            }

    async def list_changed_tools_async(
        self,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None,
        force_refresh: bool = False,
        trigger: str = "manual"
    ) -> Dict[str, Any]:
        """
        工具变更检测和处理方法（异步版本）

        Args:
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）
            force_refresh: 是否强制刷新
            trigger: 触发方式

        Returns:
            Dict: 包含变更信息的响应
        """
        if self.tools_update_monitor:
            return await self.tools_update_monitor.list_changed_tools_async(
                service_name=service_name,
                client_id=client_id,
                force_refresh=force_refresh,
                trigger=trigger
            )
        else:
            logger.warning("ToolsUpdateMonitor not available")
            return {
                "changed": False,
                "services": [],
                "trigger": trigger,
                "timestamp": self._get_timestamp(),
                "details": {"error": "ToolsUpdateMonitor not available"}
            }

    # === Resources操作支持 ===

    def list_resources(
        self,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        列出可用的资源（同步版本）

        Args:
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含资源列表的响应
        """
        return self._sync_helper.run_async(
            self.list_resources_async(service_name, client_id)
        )

    async def list_resources_async(
        self,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        列出可用的资源（异步版本）

        Args:
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含资源列表的响应
        """
        try:
            if not client_id:
                client_id = self.client_manager.global_agent_store_id

            if service_name:
                # 获取特定服务的资源
                # 从Registry获取当前活跃会话
                service_config = self.registry.get_service_config_from_cache(client_id, service_name)
                if not service_config:
                    return {
                        "success": False,
                        "error": f"Service '{service_name}' not found or not configured",
                        "data": [],
                        "service_name": service_name,
                        "timestamp": self._get_timestamp()
                    }

                async with temp_client_for_service(service_name, service_config) as client:
                    resources = await client.list_resources()
                return {
                    "success": True,
                    "data": [self._safe_model_dump(resource) for resource in resources],
                    "service_name": service_name,
                    "timestamp": self._get_timestamp(),
                    "count": len(resources)
                }
            else:
                # 获取所有服务的资源
                all_resources = {}
                services = self.registry.get_services(client_id)

                for sname in services:
                    try:
                        s_config = self.registry.get_service_config_from_cache(client_id, sname)
                        if not s_config:
                            all_resources[sname] = []
                            continue
                        async with temp_client_for_service(sname, s_config) as client:
                            resources = await client.list_resources()
                            all_resources[sname] = [self._safe_model_dump(resource) for resource in resources]
                    except Exception as e:
                        logger.warning(f"Failed to get resources from service {sname}: {e}")
                        all_resources[sname] = []

                total_count = sum(len(resources) for resources in all_resources.values())
                return {
                    "success": True,
                    "data": all_resources,
                    "timestamp": self._get_timestamp(),
                    "total_count": total_count,
                    "services_count": len(all_resources)
                }

        except Exception as e:
            logger.error(f"Error listing resources: {e}")
            return {
                "success": False,
                "error": str(e),
                "data": [],
                "timestamp": self._get_timestamp()
            }

    def list_resource_templates(
        self,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        列出可用的资源模板（同步版本）

        Args:
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含资源模板列表的响应
        """
        return self._sync_helper.run_async(
            self.list_resource_templates_async(service_name, client_id)
        )

    async def list_resource_templates_async(
        self,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        列出可用的资源模板（异步版本）

        Args:
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含资源模板列表的响应
        """
        try:
            if not client_id:
                client_id = self.client_manager.global_agent_store_id

            if service_name:
                # 获取特定服务的资源模板（使用临时client）
                service_config = self.registry.get_service_config_from_cache(client_id, service_name)
                if not service_config:
                    return {
                        "success": False,
                        "error": f"Service '{service_name}' not found or not configured",
                        "data": [],
                        "service_name": service_name,
                        "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
                    }

                async with temp_client_for_service(service_name, service_config) as client:
                    templates = await client.list_resource_templates()
                return {
                    "success": True,
                    "data": [self._safe_model_dump(template) for template in templates],
                    "service_name": service_name,
                    "timestamp": self._get_timestamp(),
                    "count": len(templates)
                }
            else:
                # 获取所有服务的资源模板
                all_templates = {}
                services = self.registry.get_services(client_id)

                for sname in services:
                    try:
                        s_config = self.registry.get_service_config_from_cache(client_id, sname)
                        if not s_config:
                            all_templates[sname] = []
                            continue
                        async with temp_client_for_service(sname, s_config) as client:
                            templates = await client.list_resource_templates()
                            all_templates[sname] = [template.model_dump() for template in templates]
                    except Exception as e:
                        logger.warning(f"Failed to get resource templates from service {sname}: {e}")
                        all_templates[sname] = []

                total_count = sum(len(templates) for templates in all_templates.values())
                return {
                    "success": True,
                    "data": all_templates,
                    "timestamp": time.strftime("%Y-%m-%d %H:%M:%S"),
                    "total_count": total_count,
                    "services_count": len(all_templates)
                }

        except Exception as e:
            logger.error(f"Error listing resource templates: {e}")
            return {
                "success": False,
                "error": str(e),
                "data": [],
                "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
            }

    def read_resource(
        self,
        uri: str,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        读取资源内容（同步版本）

        Args:
            uri: 资源URI
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含资源内容的响应
        """
        return self._sync_helper.run_async(
            self.read_resource_async(uri, service_name, client_id)
        )

    async def read_resource_async(
        self,
        uri: str,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        读取资源内容（异步版本）

        Args:
            uri: 资源URI
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含资源内容的响应
        """
        # 参数验证
        if not uri or not isinstance(uri, str):
            return {
                "success": False,
                "error": "Invalid URI parameter: URI must be a non-empty string",
                "data": None,
                "uri": uri,
                "timestamp": self._get_timestamp()
            }

        try:
            if not client_id:
                client_id = self.client_manager.global_agent_store_id

            if service_name:
                # 从特定服务读取资源（使用临时client）
                service_config = self.registry.get_service_config_from_cache(client_id, service_name)
                if not service_config:
                    return {
                        "success": False,
                        "error": f"Service '{service_name}' not found or not configured",
                        "data": None,
                        "uri": uri,
                        "service_name": service_name,
                        "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
                    }

                async with temp_client_for_service(service_name, service_config) as client:
                    content = await client.read_resource(uri)
                return {
                    "success": True,
                    "data": [self._safe_model_dump(item) for item in content],
                    "uri": uri,
                    "service_name": service_name,
                    "timestamp": self._get_timestamp(),
                    "content_count": len(content)
                }
            else:
                # TODO: 权限控制 - 后续考虑添加资源访问权限验证
                # TODO: 缓存策略 - 后续考虑添加资源内容缓存

                # 尝试从所有服务读取资源（找到第一个匹配的）
                services = self.registry.get_services(client_id)
                last_error = None

                for sname in services:
                    try:
                        s_config = self.registry.get_service_config_from_cache(client_id, sname)
                        if not s_config:
                            continue
                        async with temp_client_for_service(sname, s_config) as client:
                            content = await client.read_resource(uri)
                            return {
                                "success": True,
                                "data": [item.model_dump() for item in content],
                                "uri": uri,
                                "service_name": sname,
                                "timestamp": time.strftime("%Y-%m-%d %H:%M:%S"),
                                "content_count": len(content)
                            }
                    except Exception as e:
                        last_error = e
                        continue

                return {
                    "success": False,
                    "error": f"Resource '{uri}' not found in any service. Last error: {last_error}",
                    "data": None,
                    "uri": uri,
                    "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
                }

        except Exception as e:
            logger.error(f"Error reading resource {uri}: {e}")
            return {
                "success": False,
                "error": str(e),
                "data": None,
                "uri": uri,
                "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
            }

    # === Prompts操作支持 ===

    def list_prompts(
        self,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        列出可用的提示词（同步版本）

        Args:
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含提示词列表的响应
        """
        return self._sync_helper.run_async(
            self.list_prompts_async(service_name, client_id)
        )

    async def list_prompts_async(
        self,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        列出可用的提示词（异步版本）

        Args:
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含提示词列表的响应
        """
        try:
            if not client_id:
                client_id = self.client_manager.global_agent_store_id

            if service_name:
                # 获取特定服务的提示词
                service_config = self.registry.get_service_config_from_cache(client_id, service_name)
                if not service_config:
                    return {
                        "success": False,
                        "error": f"Service '{service_name}' not found or not configured",
                        "data": [],
                        "service_name": service_name,
                        "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
                    }

                async with temp_client_for_service(service_name, service_config) as client:
                    prompts = await client.list_prompts()
                return {
                    "success": True,
                    "data": [prompt.model_dump() for prompt in prompts],
                    "service_name": service_name,
                    "timestamp": time.strftime("%Y-%m-%d %H:%M:%S"),
                    "count": len(prompts)
                }
            else:
                # 获取所有服务的提示词
                all_prompts = {}
                services = self.registry.get_services(client_id)

                for sname in services:
                    try:
                        s_config = self.registry.get_service_config_from_cache(client_id, sname)
                        if not s_config:
                            all_prompts[sname] = []
                            continue
                        async with temp_client_for_service(sname, s_config) as client:
                            prompts = await client.list_prompts()
                            all_prompts[sname] = [prompt.model_dump() for prompt in prompts]
                    except Exception as e:
                        logger.warning(f"Failed to get prompts from service {sname}: {e}")
                        all_prompts[sname] = []

                total_count = sum(len(prompts) for prompts in all_prompts.values())
                return {
                    "success": True,
                    "data": all_prompts,
                    "timestamp": time.strftime("%Y-%m-%d %H:%M:%S"),
                    "total_count": total_count,
                    "services_count": len(all_prompts)
                }

        except Exception as e:
            logger.error(f"Error listing prompts: {e}")
            return {
                "success": False,
                "error": str(e),
                "data": [],
                "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
            }

    def get_prompt(
        self,
        name: str,
        arguments: Optional[Dict] = None,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        获取提示词内容（同步版本）

        Args:
            name: 提示词名称
            arguments: 提示词参数（可选）
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含提示词内容的响应
        """
        return self._sync_helper.run_async(
            self.get_prompt_async(name, arguments, service_name, client_id)
        )

    async def get_prompt_async(
        self,
        name: str,
        arguments: Optional[Dict] = None,
        service_name: Optional[str] = None,
        client_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        获取提示词内容（异步版本）

        Args:
            name: 提示词名称
            arguments: 提示词参数（可选）
            service_name: 特定服务名（可选）
            client_id: 特定客户端ID（可选）

        Returns:
            Dict: 包含提示词内容的响应
        """
        try:
            if not client_id:
                client_id = self.client_manager.global_agent_store_id

            if arguments is None:
                arguments = {}

            if service_name:
                # 从特定服务获取提示词（使用临时client）
                service_config = self.registry.get_service_config_from_cache(client_id, service_name)
                if not service_config:
                    return {
                        "success": False,
                        "error": f"Service '{service_name}' not found or not configured",
                        "data": None,
                        "name": name,
                        "service_name": service_name,
                        "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
                    }

                async with temp_client_for_service(service_name, service_config) as client:
                    result = await client.get_prompt(name, arguments)
                return {
                    "success": True,
                    "data": result.model_dump(),
                    "name": name,
                    "arguments": arguments,
                    "service_name": service_name,
                    "timestamp": time.strftime("%Y-%m-%d %H:%M:%S"),
                    "message_count": len(result.messages)
                }
            else:
                # TODO: 权限控制 - 后续考虑添加提示词访问权限验证
                # TODO: 缓存策略 - 后续考虑添加提示词内容缓存

                # 尝试从所有服务获取提示词（找到第一个匹配的）
                services = self.registry.get_services(client_id)
                last_error = None

                for sname in services:
                    try:
                        s_config = self.registry.get_service_config_from_cache(client_id, sname)
                        if not s_config:
                            continue
                        async with temp_client_for_service(sname, s_config) as client:
                            result = await client.get_prompt(name, arguments)
                            return {
                                "success": True,
                                "data": result.model_dump(),
                                "name": name,
                                "arguments": arguments,
                                "service_name": sname,
                                "timestamp": time.strftime("%Y-%m-%d %H:%M:%S"),
                                "message_count": len(result.messages)
                            }
                    except Exception as e:
                        last_error = e
                        continue

                return {
                    "success": False,
                    "error": f"Prompt '{name}' not found in any service. Last error: {last_error}",
                    "data": None,
                    "name": name,
                    "arguments": arguments,
                    "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
                }

        except Exception as e:
            logger.error(f"Error getting prompt {name}: {e}")
            return {
                "success": False,
                "error": str(e),
                "data": None,
                "name": name,
                "arguments": arguments,
                "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
            }



"""
服务应用服务 - 协调服务添加流程

职责:
1. 参数验证
2. 生成 client_id
3. 发布事件
4. 等待状态收敛（可选）
5. 返回结果给用户
"""

import asyncio
import logging
from dataclasses import dataclass
from typing import Dict, Any, Optional

from mcpstore.core.events.event_bus import EventBus
from mcpstore.core.events.service_events import ServiceAddRequested
from mcpstore.core.models.service import ServiceConnectionState
from mcpstore.core.utils.id_generator import ClientIDGenerator

logger = logging.getLogger(__name__)


@dataclass
class AddServiceResult:
    """服务添加结果"""
    success: bool
    service_name: str
    client_id: str
    final_state: Optional[str] = None
    error_message: Optional[str] = None
    duration_ms: float = 0.0


class ServiceApplicationService:
    """
    服务应用服务 - 用户操作的协调器
    
    职责:
    1. 参数验证
    2. 生成 client_id
    3. 发布事件
    4. 等待状态收敛（可选）
    5. 返回结果给用户
    """
    
    def __init__(
        self,
        event_bus: EventBus,
        registry: 'CoreRegistry',
        global_agent_store_id: str
    ):
        self._event_bus = event_bus
        self._registry = registry
        self._global_agent_store_id = global_agent_store_id
        
        logger.info("ServiceApplicationService initialized")
    
    async def add_service(
        self,
        agent_id: str,
        service_name: str,
        service_config: Dict[str, Any],
        wait_timeout: float = 0.0,
        source: str = "user"
    ) -> AddServiceResult:
        """
        添加服务（用户API）
        
        Args:
            agent_id: Agent ID
            service_name: 服务名称
            service_config: 服务配置
            wait_timeout: 等待超时（0表示不等待）
            source: 调用来源
            
        Returns:
            AddServiceResult: 添加结果
        """
        start_time = asyncio.get_event_loop().time()
        
        try:
            # 1. 参数验证
            self._validate_params(service_name, service_config)
            
            # 2. 生成 client_id
            client_id = self._generate_client_id(agent_id, service_name, service_config)
            
            logger.info(
                f"[ADD_SERVICE] Starting: service={service_name}, "
                f"agent={agent_id}, client_id={client_id}"
            )
            
            # 3. 发布服务添加请求事件
            event = ServiceAddRequested(
                agent_id=agent_id,
                service_name=service_name,
                service_config=service_config,
                client_id=client_id,
                source=source,
                wait_timeout=wait_timeout
            )
            
            await self._event_bus.publish(event, wait=False)
            
            # 4. 等待状态收敛（可选）
            final_state = None
            if wait_timeout > 0:
                final_state = await self._wait_for_state_convergence(
                    agent_id, service_name, wait_timeout
                )
            
            duration_ms = (asyncio.get_event_loop().time() - start_time) * 1000
            
            logger.info(
                f"[ADD_SERVICE] Completed: service={service_name}, "
                f"state={final_state}, duration={duration_ms:.2f}ms"
            )
            
            return AddServiceResult(
                success=True,
                service_name=service_name,
                client_id=client_id,
                final_state=final_state,
                duration_ms=duration_ms
            )
            
        except Exception as e:
            duration_ms = (asyncio.get_event_loop().time() - start_time) * 1000
            logger.error(f"[ADD_SERVICE] Failed: service={service_name}, error={e}", exc_info=True)
            
            return AddServiceResult(
                success=False,
                service_name=service_name,
                client_id="",
                error_message=str(e),
                duration_ms=duration_ms
            )
    
    def _validate_params(self, service_name: str, service_config: Dict[str, Any]):
        """验证参数"""
        if not service_name:
            raise ValueError("service_name cannot be empty")
        
        if not service_config:
            raise ValueError("service_config cannot be empty")
        
        # 验证必要字段
        if "command" not in service_config and "url" not in service_config:
            raise ValueError("service_config must contain 'command' or 'url'")
    
    def _generate_client_id(
        self, 
        agent_id: str, 
        service_name: str, 
        service_config: Dict[str, Any]
    ) -> str:
        """生成 client_id"""
        # 检查是否已存在
        existing_client_id = self._registry.get_service_client_id(agent_id, service_name)
        if existing_client_id:
            logger.debug(f"Using existing client_id: {existing_client_id}")
            return existing_client_id
        
        # 生成新的
        client_id = ClientIDGenerator.generate_deterministic_id(
            agent_id=agent_id,
            service_name=service_name,
            service_config=service_config,
            global_agent_store_id=self._global_agent_store_id
        )
        
        logger.debug(f"Generated new client_id: {client_id}")
        return client_id
    
    async def _wait_for_state_convergence(
        self,
        agent_id: str,
        service_name: str,
        timeout: float
    ) -> Optional[str]:
        """
        等待服务状态收敛
        
        状态收敛定义: 状态不再是 INITIALIZING
        """
        logger.debug(f"[WAIT_STATE] Waiting for {service_name} (timeout={timeout}s)")
        
        start_time = asyncio.get_event_loop().time()
        check_interval = 0.1  # 100ms
        
        while True:
            # 检查超时
            elapsed = asyncio.get_event_loop().time() - start_time
            if elapsed >= timeout:
                logger.warning(f"[WAIT_STATE] Timeout for {service_name}")
                break
            
            # 检查状态
            state = self._registry.get_service_state(agent_id, service_name)
            if state and state != ServiceConnectionState.INITIALIZING:
                logger.debug(f"[WAIT_STATE] Converged: {service_name} -> {state.value}")
                return state.value
            
            # 等待一段时间再检查
            await asyncio.sleep(check_interval)
        
        # 超时，返回当前状态
        state = self._registry.get_service_state(agent_id, service_name)
        return state.value if state else "unknown"




"""
MCPOrchestrator Service Connection Module
Service connection module - contains service connection and state management
"""

import logging
from typing import Dict, List, Any, Optional, Tuple

from fastmcp import Client

from mcpstore.core.lifecycle.health_bridge import HealthStatusBridge
from mcpstore.core.models.service import ServiceConnectionState
from .health_monitoring import HealthMonitoringMixin

logger = logging.getLogger(__name__)

class ServiceConnectionMixin(HealthMonitoringMixin):
    """Service connection mixin class"""

    async def connect_service(self, name: str, service_config: Dict[str, Any] = None, url: str = None, agent_id: str = None) -> Tuple[bool, str]:
        """
        Connect to specified service (supports local and remote services) and update cache

         缓存优先架构：优先从缓存获取配置，支持完整的服务配置

        Args:
            name: Service name
            service_config: Complete service configuration (preferred, supports all service types)
            url: Service URL (legacy parameter, only for simple HTTP services)
            agent_id: Agent ID (optional, if not provided will use global_agent_store_id)

        Returns:
            Tuple[bool, str]: (success status, message)
        """
        try:
            # 确定Agent ID
            agent_key = agent_id or self.client_manager.global_agent_store_id

            #  缓存优先：从缓存获取服务配置
            if service_config is None:
                service_config = self.registry.get_service_config_from_cache(agent_key, name)
                if not service_config:
                    return False, f"Service configuration not found in cache for {name}. This indicates a system issue."

            # 如果提供了URL，更新配置（向后兼容）
            if url:
                service_config = service_config.copy()  # 不修改原始缓存
                service_config["url"] = url

            # 判断是本地服务还是远程服务
            if "command" in service_config:
                # 本地服务：先启动进程，再连接
                return await self._connect_local_service(name, service_config, agent_key)
            else:
                # 远程服务：直接连接
                return await self._connect_remote_service(name, service_config, agent_key)

        except Exception as e:
            logger.error(f"Failed to connect service {name}: {e}")
            return False, str(e)

    async def _connect_local_service(self, name: str, service_config: Dict[str, Any], agent_id: str) -> Tuple[bool, str]:
        """连接本地服务并更新缓存"""
        try:
            # 1. 启动本地服务进程
            success, message = await self.local_service_manager.start_local_service(name, service_config)
            if not success:
                return False, f"Failed to start local service: {message}"

            #创建客户端连接
            # 本地服务通常使用 stdio 传输
            local_config = service_config.copy()

            #  修复：使用 ConfigProcessor 处理配置（与remote service保持一致）
            from mcpstore.core.configuration.config_processor import ConfigProcessor
            processed_config = ConfigProcessor.process_user_config_for_fastmcp({
                "mcpServers": {name: local_config}
            })

            if name not in processed_config.get("mcpServers", {}):
                return False, "Local service configuration processing failed"

            # 创建客户端
            client = Client(processed_config)

            # 尝试连接和获取工具列表
            try:
                async with client:
                    tools = await client.list_tools()

                    #  修复：更新Registry缓存（不缓存临时client到Registry，会话用占位句柄）
                    await self._update_service_cache(agent_id, name, client, tools, service_config)

                    # 不再缓存临时 client（async with 结束后将被关闭）
                    # self.clients[name] = client

                    #  修复：通知生命周期管理器连接成功
                    await self.lifecycle_manager.handle_health_check_result(
                        agent_id=agent_id,
                        service_name=name,
                        success=True,
                        response_time=0.0,
                        error_message=None
                    )

                    logger.info(f"Local service {name} connected successfully with {len(tools)} tools for agent {agent_id}")
                    return True, f"Local service connected successfully with {len(tools)} tools"
            except Exception as e:
                error_msg = str(e)
                logger.error(f"Failed to connect to local service {name}: {error_msg}")

                #  修复：清理资源，避免僵尸进程
                try:
                    # 停止本地服务进程
                    await self.local_service_manager.stop_local_service(name)
                    logger.debug(f"Cleaned up local service process for {name}")
                except Exception as cleanup_error:
                    logger.error(f"Failed to cleanup local service {name}: {cleanup_error}")

                # 清理客户端缓存
                if name in self.clients:
                    try:
                        client = self.clients[name]
                        if hasattr(client, 'close'):
                            await client.close()
                        del self.clients[name]
                        logger.debug(f"Cleaned up client cache for {name}")
                    except Exception as cleanup_error:
                        logger.error(f"Failed to cleanup client cache for {name}: {cleanup_error}")

                # 通知生命周期管理器连接失败
                await self.lifecycle_manager.handle_health_check_result(
                    agent_id=agent_id,
                    service_name=name,
                    success=False,
                    response_time=0.0,
                    error_message=error_msg
                )

                return False, f"Failed to connect to local service: {error_msg}"

        except Exception as e:
            error_msg = str(e)
            logger.error(f"Error connecting local service {name}: {error_msg}")

            #  修复：清理资源，避免僵尸进程
            try:
                # 停止本地服务进程
                await self.local_service_manager.stop_local_service(name)
                logger.debug(f"Cleaned up local service process for {name} after outer exception")
            except Exception as cleanup_error:
                logger.error(f"Failed to cleanup local service {name} after outer exception: {cleanup_error}")

            # 清理客户端缓存
            if name in self.clients:
                try:
                    client = self.clients[name]
                    if hasattr(client, 'close'):
                        await client.close()
                    del self.clients[name]
                    logger.debug(f"Cleaned up client cache for {name} after outer exception")
                except Exception as cleanup_error:
                    logger.error(f"Failed to cleanup client cache for {name} after outer exception: {cleanup_error}")

            # 通知生命周期管理器连接失败
            await self.lifecycle_manager.handle_health_check_result(
                agent_id=agent_id,
                service_name=name,
                success=False,
                response_time=0.0,
                error_message=error_msg
            )

            return False, error_msg

    async def _connect_remote_service(self, name: str, service_config: Dict[str, Any], agent_id: str) -> Tuple[bool, str]:
        """连接远程服务并更新缓存"""
        try:
            #  修复：使用ConfigProcessor处理配置，确保transport字段正确
            from mcpstore.core.configuration.config_processor import ConfigProcessor

            # 构造配置格式
            user_config = {"mcpServers": {name: service_config}}

            # 使用ConfigProcessor处理配置（与register_json_services保持一致）
            processed_config = ConfigProcessor.process_user_config_for_fastmcp(user_config)

            # 检查处理后的配置
            if name not in processed_config.get("mcpServers", {}):
                return False, f"Service configuration processing failed for {name}"

            # 创建新的客户端（使用处理后的配置）
            client = Client(processed_config)

            # 尝试连接
            try:
                logger.info(f" [REMOTE_SERVICE] 准备进入 async with client 上下文: {name}")
                async with client:
                    logger.info(f" [REMOTE_SERVICE] 成功进入 async with client 上下文: {name}")
                    logger.info(f" [REMOTE_SERVICE] 准备调用 client.list_tools(): {name}")
                    tools = await client.list_tools()
                    logger.info(f" [REMOTE_SERVICE] 成功获取工具列表，数量: {len(tools)}")

                    #  修复：更新Registry缓存
                    await self._update_service_cache(agent_id, name, client, tools, service_config)

                    # 不缓存临时 client（async with 结束后会自动关闭）
                    # self.clients[name] = client

                    #  修复：通知生命周期管理器连接成功
                    await self.lifecycle_manager.handle_health_check_result(
                        agent_id=agent_id,
                        service_name=name,
                        success=True,
                        response_time=0.0,
                        error_message=None
                    )

                    logger.info(f"Remote service {name} connected successfully with {len(tools)} tools for agent {agent_id}")
                    return True, f"Remote service connected successfully with {len(tools)} tools"
            except Exception as e:
                error_msg = str(e)
                logger.warning(f"Failed to connect to remote service {name}: {error_msg}")

                #  修复：清理资源，避免资源泄漏
                # 清理客户端缓存
                if name in self.clients:
                    try:
                        cached_client = self.clients[name]
                        if hasattr(cached_client, 'close'):
                            await cached_client.close()
                        del self.clients[name]
                        logger.debug(f"Cleaned up client cache for remote service {name}")
                    except Exception as cleanup_error:
                        logger.error(f"Failed to cleanup client cache for remote service {name}: {cleanup_error}")

                # 确保当前客户端也被正确关闭
                try:
                    if hasattr(client, 'close'):
                        await client.close()
                    logger.debug(f"Closed current client for remote service {name}")
                except Exception as cleanup_error:
                    logger.warning(f"Failed to close current client for remote service {name}: {cleanup_error}")

                # 通知生命周期管理器连接失败
                await self.lifecycle_manager.handle_health_check_result(
                    agent_id=agent_id,
                    service_name=name,
                    success=False,
                    response_time=0.0,
                    error_message=error_msg
                )

                return False, error_msg

        except Exception as e:
            error_msg = str(e)
            logger.error(f"Error connecting remote service {name}: {error_msg}")

            #  修复：清理资源，避免资源泄漏
            # 清理客户端缓存
            if name in self.clients:
                try:
                    cached_client = self.clients[name]
                    if hasattr(cached_client, 'close'):
                        await cached_client.close()
                    del self.clients[name]
                    logger.debug(f"Cleaned up client cache for remote service {name} after outer exception")
                except Exception as cleanup_error:
                    logger.error(f"Failed to cleanup client cache for remote service {name} after outer exception: {cleanup_error}")

            # 通知生命周期管理器连接失败
            await self.lifecycle_manager.handle_health_check_result(
                agent_id=agent_id,
                service_name=name,
                success=False,
                response_time=0.0,
                error_message=error_msg
            )

            return False, error_msg

    async def _update_service_cache(self, agent_id: str, service_name: str, client: Client, tools: List[Any], service_config: Dict[str, Any]):
        """
        更新服务缓存（工具定义、映射关系等）

        Args:
            agent_id: Agent ID
            service_name: 服务名称
            client: FastMCP客户端
            tools: 工具列表
            service_config: 服务配置
        """
        try:
            # 处理工具定义（复用register_json_services的逻辑）
            processed_tools = []
            for tool in tools:
                try:
                    original_tool_name = tool.name
                    display_name = self._generate_display_name(original_tool_name, service_name)

                    # 处理参数
                    parameters = {}
                    if hasattr(tool, 'inputSchema') and tool.inputSchema:
                        if hasattr(tool.inputSchema, 'model_dump'):
                            parameters = tool.inputSchema.model_dump()
                        elif isinstance(tool.inputSchema, dict):
                            parameters = tool.inputSchema

                    # 构建工具定义
                    tool_def = {
                        "type": "function",
                        "function": {
                            "name": original_tool_name,
                            "display_name": display_name,
                            "description": tool.description,
                            "parameters": parameters,
                            "service_name": service_name
                        }
                    }

                    processed_tools.append((display_name, tool_def))

                except Exception as e:
                    logger.error(f"Failed to process tool {tool.name}: {e}")
                    continue

            # 使用 per-agent 写锁：串行化多步缓存更新，避免并发不一致
            locks = getattr(self, 'store', None)
            agent_locks = getattr(locks, 'agent_locks', None) if locks else None
            if agent_locks is None:
                logger.warning("AgentLocks not available; proceeding without per-agent lock for cache update")
                #  优雅修复：智能清理或跳过
                existing_session = self.registry.get_session(agent_id, service_name)
                if existing_session:
                    logger.debug(f" [CACHE_UPDATE] 服务 {service_name} 已存在，执行智能清理")
                    self.registry.clear_service_tools_only(agent_id, service_name)
                else:
                    logger.debug(f" [CACHE_UPDATE] 服务 {service_name} 是新服务，跳过清理")

                # Use a stable per-service session handle (not a live client)
                session_handle = existing_session if existing_session is not None else object()
                self.registry.add_service(
                    agent_id=agent_id,
                    name=service_name,
                    session=session_handle,
                    tools=processed_tools,
                    service_config=service_config,
                    state=ServiceConnectionState.HEALTHY,
                    preserve_mappings=True
                )

                if self._is_long_lived_service(service_config):
                    self.registry.mark_as_long_lived(agent_id, service_name)

                client_id = self.registry.get_service_client_id(agent_id, service_name)
                if client_id:
                    self.registry.add_agent_client_mapping(agent_id, client_id)
                    logger.debug(f" [CLIENT_REGISTER] 注册客户端 {client_id} 到 Agent {agent_id}")
                else:
                    logger.warning(f" [CLIENT_REGISTER] 无法获取服务 {service_name} 的 Client ID")
            else:
                async with agent_locks.write(agent_id):
                    #  优雅修复：智能清理缓存，保留Agent-Client映射
                    existing_session = self.registry.get_session(agent_id, service_name)
                    if existing_session:
                        logger.debug(f" [CACHE_UPDATE] 服务 {service_name} 已存在，执行智能清理")
                        self.registry.clear_service_tools_only(agent_id, service_name)
                    else:
                        logger.debug(f" [CACHE_UPDATE] 服务 {service_name} 是新服务，跳过清理")

                    # 添加到Registry缓存（保留映射，使用稳定的占位句柄）
                    session_handle = existing_session if existing_session is not None else object()
                    self.registry.add_service(
                        agent_id=agent_id,
                        name=service_name,
                        session=session_handle,
                        tools=processed_tools,
                        service_config=service_config,
                        state=ServiceConnectionState.HEALTHY,
                        preserve_mappings=True
                    )

                    # 标记长连接服务
                    if self._is_long_lived_service(service_config):
                        self.registry.mark_as_long_lived(agent_id, service_name)

                    # 注册客户端到 Agent 客户端缓存
                    client_id = self.registry.get_service_client_id(agent_id, service_name)
                    if client_id:
                        self.registry.add_agent_client_mapping(agent_id, client_id)
                        logger.debug(f" [CLIENT_REGISTER] 注册客户端 {client_id} 到 Agent {agent_id}")
                    else:
                        logger.warning(f" [CLIENT_REGISTER] 无法获取服务 {service_name} 的 Client ID")

            # 通知生命周期管理器连接成功
            await self.lifecycle_manager.handle_health_check_result(
                agent_id=agent_id,
                service_name=service_name,
                success=True,
                response_time=0.0,  # 连接时间，可以后续优化
                error_message=None
            )

            # 将服务加入内容监控（用于运行期工具变化的兜底刷新）
            try:
                if hasattr(self, 'content_manager') and self.content_manager:
                    self.content_manager.add_service_for_monitoring(agent_id, service_name)
                    logger.debug(f"Added service '{service_name}' (agent '{agent_id}') to content monitoring")
            except Exception as e:
                logger.warning(f"Failed to add service '{service_name}' to content monitoring: {e}")

            logger.info(f"Updated cache for service '{service_name}' with {len(processed_tools)} tools for agent '{agent_id}'")

            # A+B+D: 变更后重建快照并原子发布（以全局命名域为真源）
            try:
                # 标记快照为脏；由读取方（list_tools）或此处直接触发重建均可
                if hasattr(self.registry, 'mark_tools_snapshot_dirty'):
                    self.registry.mark_tools_snapshot_dirty()
                global_agent_id = self.client_manager.global_agent_store_id
                logger.debug(f"[SNAPSHOT] connection: trigger rebuild after cache update service={service_name} agent={agent_id}")
                self.registry.rebuild_tools_snapshot(global_agent_id)
            except Exception as e:
                logger.warning(f"[SNAPSHOT] rebuild failed after cache update: {e}")

        except Exception as e:
            logger.error(f"Failed to update service cache for '{service_name}': {e}")

    def _is_long_lived_service(self, service_config: Dict[str, Any]) -> bool:
        """
        判断是否为长连接服务

        Args:
            service_config: 服务配置

        Returns:
            是否为长连接服务
        """
        # STDIO服务默认是长连接（keep_alive=True）
        if "command" in service_config:
            return service_config.get("keep_alive", True)

        # HTTP服务通常也是长连接
        if "url" in service_config:
            return True

        return False

    def _generate_display_name(self, original_tool_name: str, service_name: str) -> str:
        """
        生成用户友好的工具显示名称

        Args:
            original_tool_name: 原始工具名称
            service_name: 服务名称

        Returns:
            用户友好的显示名称
        """
        try:
            from mcpstore.core.registry.tool_resolver import ToolNameResolver
            resolver = ToolNameResolver()
            return resolver.create_user_friendly_name(service_name, original_tool_name)
        except Exception as e:
            logger.warning(f"Failed to generate display name for {original_tool_name}: {e}")
            # 回退到简单格式
            return f"{service_name}_{original_tool_name}"

    async def disconnect_service(self, url_or_name: str) -> bool:
        """从配置中移除服务并更新global_agent_store"""
        logger.info(f"Removing service: {url_or_name}")

        # 查找要移除的服务名
        name_to_remove = None
        for name, server in self.global_agent_store_config.get("mcpServers", {}).items():
            if name == url_or_name or server.get("url") == url_or_name:
                name_to_remove = name
                break

        if name_to_remove:
            # 从global_agent_store_config中移除
            if name_to_remove in self.global_agent_store_config["mcpServers"]:
                del self.global_agent_store_config["mcpServers"][name_to_remove]

            # 从配置文件中移除
            ok = self.mcp_config.remove_service(name_to_remove)
            if not ok:
                logger.warning(f"Failed to remove service {name_to_remove} from configuration file")

            # 从registry中移除
            self.registry.remove_service(name_to_remove)

            # 重新创建global_agent_store
            if self.global_agent_store_config.get("mcpServers"):
                self.global_agent_store = Client(self.global_agent_store_config)

                # 更新所有agent_clients
                for agent_id in list(self.agent_clients.keys()):
                    self.agent_clients[agent_id] = Client(self.global_agent_store_config)
                    logger.info(f"Updated client for agent {agent_id} after removing service")

            else:
                # 如果没有服务了，清除global_agent_store
                self.global_agent_store = None
                # 清除所有agent_clients
                self.agent_clients.clear()

            return True
        else:
            logger.warning(f"Service {url_or_name} not found in configuration.")
            return False

    async def refresh_services(self):
        """手动刷新所有服务连接（重新加载mcp.json）"""
        #  修复：使用统一同步管理器进行同步
        if hasattr(self, 'sync_manager') and self.sync_manager:
            await self.sync_manager.sync_global_agent_store_from_mcp_json()
        else:
            logger.warning("Sync manager not available, cannot refresh services")

    async def refresh_service_content(self, service_name: str, agent_id: str = None) -> bool:
        """手动刷新指定服务的内容（工具、资源、提示词）"""
        agent_key = agent_id or self.client_manager.global_agent_store_id
        return await self.content_manager.force_update_service_content(agent_key, service_name)

    async def is_service_healthy(self, name: str, client_id: Optional[str] = None) -> bool:
        """
        检查服务是否健康（增强版本，支持分级健康状态和智能超时）

        Args:
            name: 服务名
            client_id: 可选的客户端ID，用于多客户端环境

        Returns:
            bool: 服务是否健康（True表示healthy/warning/slow，False表示unhealthy）
        """
        result = await self.check_service_health_detailed(name, client_id)
        # 🆕 使用统一的健康状态判断逻辑
        return HealthStatusBridge.is_health_status_positive(result.status)

    def _normalize_service_config(self, service_config: Dict[str, Any]) -> Dict[str, Any]:
        """规范化服务配置，确保包含必要的字段"""
        if not service_config:
            return service_config

        # 创建配置副本
        normalized = service_config.copy()

        # 自动推断transport类型（如果未指定）
        if "url" in normalized and "transport" not in normalized:
            url = normalized["url"]
            if "/sse" in url.lower():
                normalized["transport"] = "sse"
            else:
                normalized["transport"] = "streamable-http"
            logger.debug(f"Auto-inferred transport type: {normalized['transport']} for URL: {url}")

        return normalized



"""
Compatibility proxy module.
The real implementation was moved to mcpstore.core.integration.fastmcp_integration.
This file re-exports symbols to preserve backward compatibility.
"""

from .integration.fastmcp_integration import (
    FastMCPServiceManager,
    get_fastmcp_service_manager,
)

__all__ = [
    "FastMCPServiceManager",
    "get_fastmcp_service_manager",
]




"""
MCPOrchestrator Standalone Config Module
独立配置模块 - 包含独立配置适配器
"""

import logging

logger = logging.getLogger(__name__)

class StandaloneConfigMixin:
    """独立配置混入类"""

    def _create_standalone_mcp_config(self, config_manager):
        """
        创建独立的MCP配置对象

        Args:
            config_manager: 独立配置管理器

        Returns:
            兼容的MCP配置对象
        """
        class StandaloneMCPConfigAdapter:
            """独立配置适配器 - 兼容MCPConfig接口"""

            def __init__(self, config_manager):
                self.config_manager = config_manager
                self.json_path = ":memory:"  # 表示内存配置

            def load_config(self):
                """加载配置"""
                return self.config_manager.get_mcp_config()

            def get_service_config(self, name):
                """获取服务配置"""
                return self.config_manager.get_service_config(name)

            def save_config(self, config):
                """保存配置（内存模式下不执行实际保存）"""
                logger.info("Standalone mode: config save skipped (memory-only)")
                return True

            def add_service(self, name, config):
                """添加服务"""
                self.config_manager.add_service_config(name, config)
                return True

            def remove_service(self, name):
                """移除服务"""
                # 在独立模式下，我们可以从运行时配置中移除
                services = self.config_manager.get_all_service_configs()
                if name in services:
                    del services[name]
                    logger.info(f"Removed service '{name}' from standalone config")
                    return True
                return False

        return StandaloneMCPConfigAdapter(config_manager)



"""
Hub Package Module
Hub包模块 - 管理可启动的Hub服务包
"""

import logging
import socket
from typing import List, Optional

from .process import HubProcess
from .server import HubServerGenerator
from .types import HubConfig, HubServiceInfo, HubStartMode

logger = logging.getLogger(__name__)


class HubPackage:
    """
    Hub服务包
    
    封装了一组服务的Hub配置，可以启动为独立的MCP服务进程。
    每个Hub包代表一个可部署的服务集合。
    
    特点：
    - 包含完整的服务配置信息
    - 支持多种启动模式（当前仅支持subprocess）
    - 自动端口分配和管理
    - 基于FastMCP的服务器生成
    """
    
    def __init__(self, package_name: str, services: List[HubServiceInfo], config: HubConfig):
        """
        初始化Hub服务包
        
        Args:
            package_name: 包名
            services: 服务列表
            config: Hub配置
        """
        self.package_name = package_name
        self.services = services
        self.config = config
        self._process: Optional[HubProcess] = None
        self._server_generator = HubServerGenerator()
        
        logger.info(f"HubPackage '{package_name}' created with {len(services)} services")
    
    async def start_server_async(
        self, 
        port: Optional[int] = None, 
        mode: HubStartMode = HubStartMode.SUBPROCESS
    ) -> HubProcess:
        """
        异步启动Hub服务器
        
        生成FastMCP服务器脚本并启动独立进程提供MCP服务。
        
        Args:
            port: 端口号，None则自动分配
            mode: 启动模式，目前仅支持SUBPROCESS
            
        Returns:
            HubProcess: Hub进程管理器
            
        Raises:
            ValueError: 当模式不支持时
            RuntimeError: 当启动失败时
        """
        try:
            logger.info(f"Starting Hub server '{self.package_name}' in {mode.value} mode")
            
            # 1. 端口分配
            if port is None:
                port = self._allocate_port()
            logger.debug(f"Using port: {port}")
            
            # 2. 检查启动模式
            if mode != HubStartMode.SUBPROCESS:
                raise ValueError(f"Start mode {mode.value} not implemented yet")
            
            # 3. 启动子进程
            hub_process = await self._start_subprocess(port)
            
            # 4. 缓存进程引用
            self._process = hub_process
            
            logger.info(f"Hub server '{self.package_name}' started successfully on port {port}")
            return hub_process
            
        except Exception as e:
            logger.error(f"Failed to start Hub server '{self.package_name}': {e}")
            raise RuntimeError(f"Hub server startup failed: {e}") from e
    
    def start_server(
        self, 
        port: Optional[int] = None, 
        mode: HubStartMode = HubStartMode.SUBPROCESS
    ) -> HubProcess:
        """
        同步启动Hub服务器
        
        这是start_server_async的同步版本。
        
        Args:
            port: 端口号，None则自动分配
            mode: 启动模式
            
        Returns:
            HubProcess: Hub进程管理器
        """
        # 使用asyncio运行异步方法
        import asyncio
        
        # 检查是否在异步上下文中
        try:
            loop = asyncio.get_running_loop()
            # 如果在异步上下文中，使用线程池执行
            import concurrent.futures
            with concurrent.futures.ThreadPoolExecutor() as executor:
                future = executor.submit(
                    lambda: asyncio.run(self.start_server_async(port, mode))
                )
                return future.result()
        except RuntimeError:
            # 如果不在异步上下文中，直接运行
            return asyncio.run(self.start_server_async(port, mode))
    
    def _allocate_port(self) -> int:
        """
        自动分配可用端口
        
        在3000-4000范围内查找可用端口。
        
        Returns:
            int: 可用的端口号
            
        Raises:
            RuntimeError: 当无可用端口时
        """
        # 如果配置中指定了端口，优先使用
        if self.config.port:
            if self._is_port_available(self.config.port):
                return self.config.port
            else:
                logger.warning(f"Configured port {self.config.port} is not available, auto-allocating")
        
        # 自动分配端口
        for port in range(3000, 4000):
            if self._is_port_available(port):
                logger.debug(f"Allocated port: {port}")
                return port
        
        raise RuntimeError("No available ports in range 3000-4000")
    
    def _is_port_available(self, port: int) -> bool:
        """
        检查端口是否可用
        
        Args:
            port: 端口号
            
        Returns:
            bool: 端口是否可用
        """
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.bind(('localhost', port))
                return True
        except OSError:
            return False
    
    async def _start_subprocess(self, port: int) -> HubProcess:
        """
        通过子进程启动Hub服务器
        
        Args:
            port: 端口号
            
        Returns:
            HubProcess: Hub进程管理器
        """
        try:
            # 1. 生成服务器脚本和配置
            script_file, config_file = await self._server_generator.generate_server_files_async(
                package_name=self.package_name,
                services=self.services,
                config=self.config,
                port=port
            )
            
            # 2. 启动子进程
            process = await self._server_generator.start_subprocess_async(
                script_file=script_file,
                config_file=config_file,
                port=port
            )
            
            # 3. 创建进程管理器
            hub_process = HubProcess(
                package_name=self.package_name,
                process=process,
                port=port,
                config_file=config_file,
                script_file=script_file,
                services=self.services
            )
            
            # 4. 等待服务器启动
            await hub_process.wait_for_startup()
            
            return hub_process
            
        except Exception as e:
            logger.error(f"Failed to start subprocess for Hub '{self.package_name}': {e}")
            raise
    
    @property
    def is_running(self) -> bool:
        """
        检查Hub是否正在运行
        
        Returns:
            bool: 是否正在运行
        """
        return self._process is not None and self._process.is_running
    
    @property
    def process(self) -> Optional[HubProcess]:
        """
        获取当前的Hub进程
        
        Returns:
            Optional[HubProcess]: Hub进程管理器，如果未启动则为None
        """
        return self._process
    
    def get_summary(self) -> dict:
        """
        获取Hub包摘要信息
        
        Returns:
            dict: 包含包名、服务数量、配置等信息的摘要
        """
        return {
            "package_name": self.package_name,
            "services_count": len(self.services),
            "service_names": [s.name for s in self.services],
            "config": {
                "name": self.config.name,
                "description": self.config.description,
                "context_type": self.config.context_type,
                "target_id": self.config.target_id,
                "basic_routing": self.config.basic_routing
            },
            "is_running": self.is_running,
            "process_info": self._process.get_info() if self._process else None
        }



#!/usr/bin/env python3
"""
Intelligent Caching and Performance Optimization
Tool result caching, service discovery caching, intelligent prefetching, connection pool management
"""

import asyncio
import logging
import pickle
from collections import OrderedDict, defaultdict
from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Dict, List, Any, Optional

logger = logging.getLogger(__name__)

class CacheStrategy(Enum):
    """Cache strategies"""
    LRU = "lru"           # Least Recently Used
    LFU = "lfu"           # Least Frequently Used
    TTL = "ttl"           # Time To Live
    ADAPTIVE = "adaptive"  # Adaptive

@dataclass
class CacheEntry:
    """Cache entry"""
    key: str
    value: Any
    created_at: datetime
    last_accessed: datetime
    access_count: int = 0
    ttl: Optional[int] = None  # Time to live (seconds)
    size: int = 0  # Data size (bytes)

@dataclass
class CacheStats:
    """Cache statistics"""
    hits: int = 0
    misses: int = 0
    evictions: int = 0
    total_size: int = 0
    entry_count: int = 0
    
    @property
    def hit_rate(self) -> float:
        total = self.hits + self.misses
        return self.hits / total if total > 0 else 0.0

class LRUCache:
    """LRU 缓存实现"""
    
    def __init__(self, max_size: int = 1000, max_memory: int = 100 * 1024 * 1024):  # 100MB
        self.max_size = max_size
        self.max_memory = max_memory
        self._cache: OrderedDict[str, CacheEntry] = OrderedDict()
        self._stats = CacheStats()
    
    def get(self, key: str) -> Optional[Any]:
        """获取缓存值"""
        if key in self._cache:
            entry = self._cache[key]
            
            # 检查 TTL
            if entry.ttl and (datetime.now() - entry.created_at).total_seconds() > entry.ttl:
                self._evict(key)
                self._stats.misses += 1
                return None
            
            # 更新访问信息
            entry.last_accessed = datetime.now()
            entry.access_count += 1
            
            # 移到末尾（最近使用）
            self._cache.move_to_end(key)
            
            self._stats.hits += 1
            return entry.value
        
        self._stats.misses += 1
        return None
    
    def put(self, key: str, value: Any, ttl: Optional[int] = None):
        """存储缓存值"""
        # 计算数据大小
        size = self._calculate_size(value)
        
        # 检查是否需要清理空间
        while (len(self._cache) >= self.max_size or 
               self._stats.total_size + size > self.max_memory):
            if not self._cache:
                break
            self._evict_lru()
        
        # 创建缓存条目
        entry = CacheEntry(
            key=key,
            value=value,
            created_at=datetime.now(),
            last_accessed=datetime.now(),
            ttl=ttl,
            size=size
        )
        
        # If key already exists, update statistics
        if key in self._cache:
            old_entry = self._cache[key]
            self._stats.total_size -= old_entry.size
        
        self._cache[key] = entry
        self._stats.total_size += size
        self._stats.entry_count = len(self._cache)
    
    def _evict_lru(self):
        """Evict least recently used entry"""
        if self._cache:
            key, entry = self._cache.popitem(last=False)
            self._stats.total_size -= entry.size
            self._stats.evictions += 1
            logger.debug(f"Evicted LRU cache entry: {key}")
    
    def _evict(self, key: str):
        """Evict specified entry"""
        if key in self._cache:
            entry = self._cache.pop(key)
            self._stats.total_size -= entry.size
            self._stats.evictions += 1
    
    def _calculate_size(self, value: Any) -> int:
        """Calculate size of value"""
        try:
            return len(pickle.dumps(value))
        except:
            return len(str(value).encode('utf-8'))
    
    def clear(self):
        """Clear cache"""
        self._cache.clear()
        self._stats = CacheStats()
    
    def get_stats(self) -> CacheStats:
        """Get cache statistics"""
        self._stats.entry_count = len(self._cache)
        return self._stats

# ToolResultCache 类已移除 - 不再支持工具结果缓存功能

class ServiceDiscoveryCache:
    """Service discovery cache"""

    def __init__(self, ttl: int = 300):  # 5 minutes
        self.cache = LRUCache(max_size=100)
        self.ttl = ttl
    
    def get_service_info(self, service_name: str) -> Optional[Dict[str, Any]]:
        """Get service information"""
        return self.cache.get(f"service:{service_name}")
    
    def cache_service_info(self, service_name: str, service_info: Dict[str, Any]):
        """缓存服务信息"""
        self.cache.put(f"service:{service_name}", service_info, self.ttl)
    
    def get_tools_for_service(self, service_name: str) -> Optional[List[Dict[str, Any]]]:
        """获取服务的工具列表"""
        return self.cache.get(f"tools:{service_name}")
    
    def cache_tools_for_service(self, service_name: str, tools: List[Dict[str, Any]]):
        """缓存服务的工具列表"""
        self.cache.put(f"tools:{service_name}", tools, self.ttl)

class PrefetchManager:
    """智能预取管理器（简化版，移除工具使用模式记录）"""

    def __init__(self):
        self._prefetch_queue: asyncio.Queue = asyncio.Queue()
        self._running = False


    
    async def start_prefetch_worker(self):
        """启动预取工作器"""
        self._running = True
        while self._running:
            try:
                prefetch_task = await asyncio.wait_for(
                    self._prefetch_queue.get(), timeout=1.0
                )
                await self._execute_prefetch(prefetch_task)
            except asyncio.TimeoutError:
                continue
            except Exception as e:
                logger.error(f"Prefetch error: {e}")
    
    def stop_prefetch_worker(self):
        """停止预取工作器"""
        self._running = False
    
    async def _execute_prefetch(self, task: Dict[str, Any]):
        """执行预取任务"""
        # 这里可以实现具体的预取逻辑
        logger.debug(f"Executing prefetch task: {task}")

class ConnectionPoolManager:
    """连接池管理器"""
    
    def __init__(self, max_connections: int = 50):
        self.max_connections = max_connections
        self._pools: Dict[str, asyncio.Queue] = {}
        self._connection_counts: Dict[str, int] = defaultdict(int)
        self._lock = asyncio.Lock()
    
    async def get_connection(self, service_name: str) -> Optional[Any]:
        """获取连接"""
        async with self._lock:
            if service_name not in self._pools:
                self._pools[service_name] = asyncio.Queue(maxsize=self.max_connections)
            
            pool = self._pools[service_name]
            
            try:
                # 尝试从池中获取连接
                connection = pool.get_nowait()
                logger.debug(f"Reused connection for service {service_name}")
                return connection
            except asyncio.QueueEmpty:
                # 创建新连接
                if self._connection_counts[service_name] < self.max_connections:
                    connection = await self._create_connection(service_name)
                    if connection:
                        self._connection_counts[service_name] += 1
                        logger.debug(f"Created new connection for service {service_name}")
                        return connection
                
                logger.warning(f"Connection pool exhausted for service {service_name}")
                return None
    
    async def return_connection(self, service_name: str, connection: Any):
        """归还连接"""
        if service_name in self._pools:
            pool = self._pools[service_name]
            try:
                pool.put_nowait(connection)
                logger.debug(f"Returned connection for service {service_name}")
            except asyncio.QueueFull:
                # 池已满，关闭连接
                await self._close_connection(connection)
                self._connection_counts[service_name] -= 1
    
    async def _create_connection(self, service_name: str) -> Optional[Any]:
        """创建连接（需要子类实现）"""
        # 这里应该根据服务类型创建相应的连接
        return None
    
    async def _close_connection(self, connection: Any):
        """关闭连接（需要子类实现）"""
        pass

class PerformanceOptimizer:
    """性能优化器"""

    def __init__(self):
        # 移除工具结果缓存功能
        self.service_cache = ServiceDiscoveryCache()
        self.prefetch_manager = PrefetchManager()
        self.connection_pool = ConnectionPoolManager()
        self._metrics: Dict[str, Any] = defaultdict(list)

    def enable_caching(self, patterns: Dict[str, int] = None):
        """启用缓存（工具结果缓存已移除，仅保留服务发现缓存）"""
        logger.info("Tool result caching has been removed. Only service discovery caching is available.")
        return True
    
    def record_tool_execution(self, tool_name: str, execution_time: float, success: bool):
        """记录工具执行指标"""
        self._metrics[tool_name].append({
            "execution_time": execution_time,
            "success": success,
            "timestamp": datetime.now()
        })
        
        # 保持最近的100条记录
        if len(self._metrics[tool_name]) > 100:
            self._metrics[tool_name].pop(0)
    
    def get_performance_summary(self) -> Dict[str, Any]:
        """获取性能摘要（已移除工具结果缓存统计）"""
        service_cache_stats = self.service_cache.cache.get_stats()

        return {
            "service_cache": {
                "hit_rate": service_cache_stats.hit_rate,
                "entries": service_cache_stats.entry_count
            },
            "connection_pools": {
                service: count for service, count in self.connection_pool._connection_counts.items()
            },
            "tool_metrics": {
                tool: {
                    "avg_execution_time": sum(m["execution_time"] for m in metrics) / len(metrics),
                    "success_rate": sum(1 for m in metrics if m["success"]) / len(metrics),
                    "total_calls": len(metrics)
                }
                for tool, metrics in self._metrics.items() if metrics
            }
        }

# 全局实例
_global_performance_optimizer = None

def get_performance_optimizer() -> PerformanceOptimizer:
    """获取全局性能优化器"""
    global _global_performance_optimizer
    if _global_performance_optimizer is None:
        _global_performance_optimizer = PerformanceOptimizer()
    return _global_performance_optimizer



# src/mcpstore/adapters/llamaindex_adapter.py
from __future__ import annotations

from typing import List, TYPE_CHECKING

from .common import enhance_description, create_args_schema, build_sync_executor

# TYPE_CHECKING to avoid runtime circular imports
if TYPE_CHECKING:
    from ..core.context.base_context import MCPStoreContext
    from ..core.models.tool import ToolInfo




class LlamaIndexAdapter:
    """
    Adapter from MCPStore ToolInfo -> LlamaIndex FunctionTool list.
    """
    def __init__(self, context: 'MCPStoreContext'):
        self._context = context

    def list_tools(self) -> List[object]:
        return self._context._sync_helper.run_async(self.list_tools_async())

    async def list_tools_async(self) -> List[object]:
        try:
            from llama_index.core.tools import FunctionTool
        except Exception as e:
            raise ImportError("LlamaIndex adapter requires 'llama-index' (llama_index). Install: pip install llama-index") from e

        mcp_tools: List['ToolInfo'] = await self._context.list_tools_async()
        tools: List[object] = []
        for t in mcp_tools:
            args_schema = create_args_schema(t)
            sync_fn = build_sync_executor(self._context, t.name, args_schema)
            desc = enhance_description(t)
            # LlamaIndex primarily accepts sync functions; name/description can be set via from_defaults
            li_tool = FunctionTool.from_defaults(fn=sync_fn, name=t.name, description=desc)
            tools.append(li_tool)
        return tools




"""
MCPStore API 服务 - 已弃用
请使用 api_app.py 中的 create_app() 函数

此文件仅保留用于向后兼容，将在未来版本中移除
"""

import logging

# 导入新的应用工厂
from .api_app import create_app

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

# 为了向后兼容，创建应用实例
# 注意：这个文件已被弃用，请使用 api_app.py
logger.warning("app.py is deprecated. Please use api_app.create_app() instead.")

# 创建应用实例（委托给新的工厂函数）
app = create_app()

# 添加弃用警告的中间件
@app.middleware("http")
async def deprecation_warning(request, call_next):
    """添加弃用警告"""
    if request.url.path.startswith("/docs") or request.url.path.startswith("/openapi"):
        # API 文档路径，添加警告头部
        response = await call_next(request)
        response.headers["X-Deprecation-Warning"] = "app.py is deprecated. Please migrate to api_app.create_app()"
        return response
    return await call_next(request)
